{
  This demo draws rectangles around the current item.
  There are two options:
  - drawing rectangle around the item part (when a text item occupies
    several lines, each line is a part of this item)
  - drawing rectangle around the whole item
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, SclRView, RVEdit;

type
  TForm1 = class(TForm)
    SRichViewEdit1: TSRichViewEdit;
    Panel1: TPanel;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure CaretMove1(Sender: TObject);
    procedure CaretMove2(Sender: TObject);
    procedure PaintPage1(Sender: TObject; PageNo: Integer;
      PageRect, R: TRect; Canvas: TCanvas; Prepaint, Printing: Boolean);
    procedure PaintPage2(Sender: TObject; PageNo: Integer;
      PageRect, R: TRect; Canvas: TCanvas; Prepaint, Printing: Boolean);
    procedure RadioButton1Click(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
  private
    { Private declarations }
    function CalcRect1(PageNo: Integer; ClientCoords: Boolean; var R: TRect): Boolean;
    function CalcRect2(PageNo: Integer; ClientCoords: Boolean; var R: TRect): Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  OldRepaintRect : TRect;

implementation

{$R *.DFM}
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  OldRepaintRect := Bounds(0, 0, 0, 0);
end;
{------------------------------------------------------------------------------}
// Calculating rectangle around the current item, option 1:
//   rectangle around the item part.
// If ClientCoords=False:
//  the rectangle is calculated in 100% zooming, relative to the PageNo-th page
// If ClientCoords=True:
//  the rectangle is calculated in client coordinates, PageNo is ignored; rectangle
//  is expanded by 5 pixels
function TForm1.CalcRect1(PageNo: Integer; ClientCoords: Boolean; var R: TRect): Boolean;
var
  ItemPart, FirstPageNo, LastPageNo: Integer;
  ItemPartRect, PageRect: TRect;
begin
  Result := False;
  if not ClientCoords then begin // checking if the current item is on this page
    if PageNo<>SRichViewEdit1.CurrentPage then
      exit;
  end;
  // receiving the position of the item part containing caret
  with SRichViewEdit1.RichViewEdit.TopLevelEditor do begin
    ItemPart := RVData.GetItemPart(CurItemNo, OffsetInCurItem);
    SRichViewEdit1.GetItemBounds100(RVData, CurItemNo, ItemPartRect,
      FirstPageNo, LastPageNo, ItemPart);
  end;
  if ClientCoords then begin
    // expanding rectangle by 5 pixels
    InflateRect(ItemPartRect, 5, 5);
    // converting from { zoom=100%, relative to page } to { current zoom, client coordinates }
    PageRect := SRichViewEdit1.GetPageClientRect(FirstPageNo);
    ItemPartRect := SRichViewEdit1.ZoomRectIn(ItemPartRect, SRichViewEdit1.ViewProperty.ZoomPercent * 0.01);
    OffsetRect(ItemPartRect, PageRect.Left, PageRect.Top);
    end
  else begin
    // if not on this page, exiting
    if FirstPageNo<>PageNo then
      exit;
  end;
  R := ItemPartRect;
  Result := True;
end;
{------------------------------------------------------------------------------}
// Calculating rectangle around the current item, option 2:
//   rectangle around the item
// If ClientCoords=False:
//  the rectangle is calculated in 100% zooming, relative to the PageNo-th page
// If ClientCoords=True:
//  the rectangle is calculated in client coordinates, PageNo is ignored; rectangle
//  is expanded by 5 pixels
function TForm1.CalcRect2(PageNo: Integer; ClientCoords: Boolean; var R: TRect): Boolean;
var
     CurItemNo, FirstItemNo, LastItemNo, Offs, ItemPart : Integer;
     FirstPageNo, LastPageNo : Integer;
     ItemPartRect, PageRect : TRect;
     rve: TCustomRichViewEdit;
begin
  Result := False;
  if not ClientCoords then begin  // checking if the current item is on this page
    // receiving the range of items on the page
    SRichViewEdit1.GetPageStartItemNo(PageNo, FirstItemNo, Offs);
    SRichViewEdit1.GetPageLastItemNo(PageNo, LastItemNo, Offs);
    // checking if the current item may be on this page
    // this check is not complete: it checks only the root editor, but the
    // caret may be in a table cell
    CurItemNo := SRichViewEdit1.RichViewEdit.CurItemNo;
    If (CurItemNo < FirstItemNo) or (CurItemNo > LastItemNo) then
      exit;
  end;
  // Calculating the rectangle around the current item on this page
  rve := SRichViewEdit1.RichViewEdit.TopLevelEditor;
  CurItemNo := rve.CurItemNo;
  ItemPart := 0;
  R := Bounds(0, 0, 0, 0);
  while SRichViewEdit1.GetItemBounds100(rve.RVData, CurItemNo,
    ItemPartRect, FirstPageNo, LastPageNo, ItemPart) do begin
    if ClientCoords then begin
      // expanding rectangle by 5 pixels
      InflateRect(ItemPartRect, 5, 5);
      // converting from { zoom=100%, relative to page } to { current zoom, client coordinates }
      PageRect := SRichViewEdit1.GetPageClientRect(FirstPageNo);
      ItemPartRect := SRichViewEdit1.ZoomRectIn(ItemPartRect, SRichViewEdit1.ViewProperty.ZoomPercent * 0.01);
      OffsetRect(ItemPartRect, PageRect.Left, PageRect.Top);
      // combining
      UnionRect(R, R, ItemPartRect);
      end
    else if (FirstPageNo = PageNo) then
      // combining, if on this page
      UnionRect(R, R, ItemPartRect);
    inc(ItemPart);
  end;
  Result := True;  
end;
{------------------------------------------------------------------------------}
// OnPaintPage, option 1: Drawing rectangle around the item part
procedure TForm1.PaintPage1(Sender: TObject; PageNo: Integer;
  PageRect, R: TRect; Canvas: TCanvas; Prepaint, Printing: Boolean);
var ItemRect: TRect;
begin
  // receiving coordinates in zoom=100%, relative to page
  if not CalcRect1(PageNo, False, ItemRect) then
    exit;
  // converting to "client" coordinates in zoom=100%
  OffsetRect(ItemRect, PageRect.Left, PageRect.Top);
  // drawing
  Canvas.Pen.Color := clRed;
  Canvas.Brush.Style := bsClear;
  with ItemRect do
    Canvas.Rectangle(Left, Top, Right, Bottom);
end;
{------------------------------------------------------------------------------}
// OnPaintPage, option 2: Drawing rectangle around the item
procedure TForm1.PaintPage2(Sender: TObject; PageNo: Integer;
  PageRect, R: TRect; Canvas: TCanvas; Prepaint, Printing: Boolean);
var ItemRect: TRect;
begin
  // receiving coordinates in zoom=100%, relative to page
  if not CalcRect2(PageNo, False, ItemRect) then
    exit;
  // converting to "client" coordinates in zoom=100%
  OffsetRect(ItemRect, PageRect.Left, PageRect.Top);
  // drawing  
  Canvas.Pen.Color := clRed;
  Canvas.Brush.Style := bsClear;
  with ItemRect do
    Canvas.Rectangle(Left, Right, Top, Bottom);
end;
{------------------------------------------------------------------------------}
// OnCaretMove for option 1
procedure TForm1.CaretMove1(Sender: TObject);
var ARect, ItemRect : TRect;
begin
  // receiving client coordinates of the rectangle, expanded by 5 pixels
  if not CalcRect1(0, True, ItemRect) then
    exit;
  // redrawing, if necessary
  if not EqualRect(ItemRect, OldRepaintRect) then begin
    UnionRect(ARect, ItemRect, OldRepaintRect);
    SRichViewEdit1.UpdateBufferAt(ARect);
    OldRepaintRect := ItemRect;
  end;
end;
{------------------------------------------------------------------------------}
// OnCaretMove for option 2
procedure TForm1.CaretMove2(Sender: TObject);
var ARect, ItemRect : TRect;
begin
  // receiving client coordinates of the rectangle, expanded by 5 pixels
  if not CalcRect2(0, True, ItemRect) then
    exit;
  // redrawing, if necessary    
  if not EqualRect(ItemRect, OldRepaintRect) then begin
    UnionRect(ARect, ItemRect, OldRepaintRect);
    SRichViewEdit1.UpdateBufferAt(ARect);
    OldRepaintRect := ItemRect;
  end;
end;
{------------------------------------------------------------------------------}
// Option 1
procedure TForm1.RadioButton1Click(Sender: TObject);
begin
  SRichViewEdit1.OnCaretMove := CaretMove1;
  SRichViewEdit1.OnPaintPage := PaintPage1;
  SRichViewEdit1.UpdateBuffer;
end;
{------------------------------------------------------------------------------}
// Option 2
procedure TForm1.RadioButton2Click(Sender: TObject);
begin
  SRichViewEdit1.OnCaretMove := CaretMove2;
  SRichViewEdit1.OnPaintPage := PaintPage2;
  SRichViewEdit1.UpdateBuffer;
end;

end.
