unit SRVMediaPlayer;

interface
{$I RV_Defs.inc}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  SRVControl, DShow, RVStyle, ActiveX;

const
  WM_SRVMEDIAPLAYER_NOTIFY = WM_USER + 1988;
  // Unique event identifier
  dsEventsOn       = 0;
  dsEventsOff      = 1;

type
  { Property pages.<br>See also: @link(ShowFilterPropertyPage), @link:(HaveFilterPropertyPage).}
  TPropertyPage = (
    ppDefault,       // Simple property page.
    ppVFWCapDisplay, // Capture Video source dialog box.
    ppVFWCapFormat,  // Capture Video format dialog box.
    ppVFWCapSource,  // Capture Video source dialog box.
    ppVFWCompConfig, // Compress Configure dialog box.
    ppVFWCompAbout   // Compress About Dialog box.
  );

  TCCTextService = ( stsNone, stsCaption1, stsCaption2, stsText1, stsText2,
    stsXDS, stsDefChannel, stsInvalid);

  TBGModeSubtitle  = (bgmsOpaque, bgmsTransparent);

  TRatioModes  = (rmStretched, rmLetterBox, rmCrop);

  TSRVVideoFormat = (vfNone, vfNTSC_M, vfNTSC_M_J, vfNTSC_433, vfPAL_B, vfPAL_D,
                     vfPAL_G, vfPAL_H, vfPAL_I, vfPAL_M, vfPAL_N, vfPAL_60,
                     vfSECAM_B, vfSECAM_D, vfSECAM_G, vfSECAM_H, vfSECAM_K,
                     vfSECAM_K1, vfSECAM_L, vfSECAM_L1);
  TSRVVideoFormats = set of TSRVVideoFormat;
  TRVMPModes = (rvmpNotReady, rvmpStopped, rvmpPlaying, rvmpPaused, rvmpOpen);
  TSRVTunerModeType = (tmDefault, tmTV, tmFMRadio, tmAMRadio, tmDSS);
  TSRVMPOnChange = procedure(Sender : TObject; Pos, MaxPos : Double) of object;
  TCursorVisibleEvent = procedure(Sender: TObject; Visible : Boolean) of object;

  TSRVMediaPlayer = class(TSRVRenderControl)
  private
    { Private declarations }
    Filter : TFilter;
    AudioInFilter : TFilter;
    AudioOutFilter : TFilter;
    SampleGrabberVideo : TSampleGrabber;
    SampleGrabberAudio : TSampleGrabber;

    MyMediaControl : IMediaControl;
    MyMediaPosition : IMediaPosition;
    MyBaseFilter : IBaseFilter;
    MyMediaSeeking: IMediaSeeking;
    MyVideoWindow : IVideoWindow;
    MyBasicVideo : IBasicVideo;
    MyBasicAudio : IBasicAudio;
    MyMediaEvent  : IMediaEvent;
    MyMediaEventEx  : IMediaEventEx;
    MyFilterGraph : IFilterGraph;
    MyAMTuner : IAMTuner;
    MyAMTVTuner : IAMTVTuner;
    MyAMCrossbar : IAMCrossbar;
    MyAMVideoControl : IAMVideoControl;
    MyAMVideoProcAmp : IAMVideoProcAmp;
    MyAMTVAudio : IAMTVAudio;
    MyAMStreamSelect : IAMStreamSelect;
    MyIAMWstDecoder : IAMWstDecoder;
    Line21 : IAMLine21Decoder;

    VideoDevice : TSysDevEnum;
    AudioInDevice : TSysDevEnum;
    AudioOutDevice : TSysDevEnum;
    VideoMediaTypes : TEnumMediaType;
    AudioMediaInTypes : TEnumMediaType;
    AudioMediaOutTypes : TEnumMediaType;
    FVolume : Integer;
    FIndexSoundChannal : Integer;
    FIndexSubtitleLanguage : Integer;
    FAllSound : Boolean;
    FState : TRVMPModes;
    FFileName : String;
    FFullScreen : Boolean;
    FDVDSpeed : Double;
    DVDCurrentPos : Cardinal;
    FZoom : Integer;
    FIdleCursor : Integer;
    MousePos : TPoint;
    CursorMov : DWord;
    IsHidden : Bool;
    DefaultLeft : Integer;
    DefaultTop : Integer;
    DefaultWidth : Integer;
    DefaultHeight : Integer;
    FOnChange : TSRVMPOnChange;
    FOnBufferVideo : TOnBuffer;
    FOnBufferAudio : TOnBuffer;
    FOnCursorVisible : TCursorVisibleEvent;
    FOnDVDCurrentHMSFTime : TOnDVDCurrentHMSFTime;
  protected
    { Protected declarations }
    IndexAudioInFormat : Integer;
    IndexInputLines : Integer;
    InputLines : TStringList;

    procedure Loaded; override;
    function GetRenderVisible: Boolean; override;
    procedure SetRenderVisible(Value : Boolean); override;
    procedure SetDisplay(Value : TWinControl); override;
    procedure SetDisplayRect(Value : TRect); override;
    procedure SetSourceRect(Value : TRect); override;
    procedure SetDestinationRect(Value : TRect); override;
    procedure SetFullScreen(Value : Boolean);
    procedure MediaPlayerNotify(var Msg : TMessage); message WM_SRVMEDIAPLAYER_NOTIFY;
    procedure SetRotation(Value : TRVDocRotation); override;
    procedure Timer;
    procedure HookDVDCurrentHMSFTime(Sender: TObject;
      HMSFTimeCode: TDVD_HMSF_TimeCode; TimeCode: TDVD_TimeCode);

    function  GetDVDVolumeCount : Integer;
    function  GetDVDCurrentVolume : Integer;
    function  GetDVDSide : TDVD_Disc_Side;
    function  GetDVDTitleCount : Integer;
    function  GetVolume : Integer;
    procedure SetVolume(Value : Integer);
    function  GetBalance : Integer;
    procedure SetBalance(Value : Integer);
    function  GetGraphMode : TGraphMode;
    procedure SetGraphMode(Value : TGraphMode);
    function  GetVideoSpeed : Double;
    procedure SetVideoSpeed(Value : Double);
    function  GetVideoLength : Double;
    function  GetVideoPosition : Double;
    procedure SetVideoPosition(Value : Double);
    function  GetTunerMode : TSRVTunerModeType;
    procedure SetTunerMode(Value : TSRVTunerModeType);
    function  GetTunerCountry : Integer;
    procedure SetTunerCountry(Value : Integer);
    function  GetTunerType : TTunerInputType;
    procedure SetTunerType(Value : TTunerInputType);
    function  GetTunerChannel : Integer;
    procedure SetTunerChannel(Value : Integer);
    function  GetTunerChannelMin : Integer;
    function  GetTunerChannelMax : Integer;
    function  GetTunerSignalPresent : Boolean;
    function  GetVideoFrequency : Integer;
    function  GetAudioFrequency : Integer;
    procedure SetZoom(Value : Integer);
    procedure EnabledAllSoundStream(Volume : Integer; Enabled : Boolean);
    function  GetVolumeSoundStream(Index : Integer) : Integer;
    procedure SetSoundStream(Index, Volume : Integer);
    function  GetSoundStreamsCount : Integer;
    function  GetIndexSoundChannal : Integer;
    procedure SetIndexSoundChannal(Value : Integer);
    procedure SetAllSound(Value : Boolean);
    // Language
    function  GetLanguagesCount: Integer;
    function  GetIndexLanguage: Integer;
    procedure SetIndexLanguage(Value: Integer);
    // Subtitle
    function  GetShowSubtitles : Boolean;
    procedure SetShowSubtitles(Value : Boolean);
    function  GetFileNameSubtitles : String;
    procedure SetFileNameSubtitles(Value : String);
    function  GetSubtitleLanguagesCount : Integer;
    function  GetIndexSubtitleLanguage : Integer;
    procedure SetIndexSubtitleLanguage(Value : Integer);
    function  GetFlipTextSubtitles : Boolean;
    procedure SetFlipTextSubtitles(Value : Boolean);
    function  GetFlipPictureSubtitles : Boolean;
    procedure SetFlipPictureSubtitles(Value : Boolean);
    function  GetRectSubtitles : TNormalizedRect;
    procedure SetRectSubtitles(Value : TNormalizedRect);
    function  GetColorFormatSubtitles : Integer;
    procedure SetColorFormatSubtitles(Value : Integer);
    function  GetOSDSubtitle : Boolean;
    procedure SetOSDSubtitle(Value : Boolean);
    // Close Caption Text
    function  GetShowCCText : Boolean;
    procedure SetShowCCText(Value : Boolean);
    function  GetBackgroundColorCCText: TColor;
    procedure SetBackgroundColorCCText(Value: TColor);
    function  GetBackgroundModeCCText: TBGModeSubtitle;
    procedure SetBackgroundModeCCText(Value: TBGModeSubtitle);
    function  GetServiceCCText: TCCTextService;
    procedure SetServiceCCText(Value: TCCTextService);

    function  GetOnDVDSubpictureStreamChange : TOnDVDSubpictureStreamChange;
    procedure SetOnDVDSubpictureStreamChange(Value : TOnDVDSubpictureStreamChange);
    function  GetOnDVDAudioStreamChange : TOnDVDAudioStreamChange;
    procedure SetOnDVDAudioStreamChange(Value : TOnDVDAudioStreamChange);
    function  GetOnDVDTitleChange : TOnDVDTitleChange;
    procedure SetOnDVDTitleChange(Value : TOnDVDTitleChange);
    function  GetOnDVDChapterStart : TOnDVDChapterStart;
    procedure SetOnDVDChapterStart(Value : TOnDVDChapterStart);
    function  GetOnDVDAngleChange : TOnDVDChange;
    procedure SetOnDVDAngleChange(Value : TOnDVDChange);
    function  GetOnDVDButtonChange : TOnDVDChange;
    procedure SetOnDVDButtonChange(Value : TOnDVDChange);
    function  GetOnDVDPlaybackRateChange : TOnDVDPlaybackRateChange;
    procedure SetOnDVDPlaybackRateChange(Value : TOnDVDPlaybackRateChange);
    function  GetOnDVDCurrentTime : TOnDVDCurrentTime;
    procedure SetOnDVDCurrentTime(Value : TOnDVDCurrentTime);
    function  GetOnDVDKaraokeMode : TOnDVDKaraokeMode;
    procedure SetOnDVDKaraokeMode(Value : TOnDVDKaraokeMode);
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure MyIdleHandler(Sender: TObject; var Done: Boolean);
    function  FindFilter(Cid: TGuid): IBaseFilter;
    function  ShowFilterPropertyPage(parent: THandle; Filter: IBaseFilter;
      PropertyPage: TPropertyPage = ppDefault): HRESULT;

    function  DVDGetImage: HBITMAP;
    function  HasConfigDialogSubtitles(iSelected : Integer) : Boolean;
    procedure ShowConfigDialogSubtitles(iSelected : Integer; hWndParent : THandle);
    property RectSubtitles : TNormalizedRect read GetRectSubtitles write SetRectSubtitles;
  public
    { Public declarations }
    FilterGraph : TFilterGraph;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure WndProc(var Message: TMessage); override;
    procedure GetCurrentImageToStream(Stream : TStream);
    function GetCurrentImage : TBitmap;
    procedure GetCurrentImageToFile(FileName : String);
    function  GetEnumVideoDevice(Items: TStrings) : Boolean;
    function  GetEnumAudioInDevice(Items: TStrings) : Boolean;
    function  GetEnumAudioOutDevice(Items: TStrings) : Boolean;
    function  GetVideoDeviceByIndex(Index : Integer) : IMoniker;
    function  GetAudioInDeviceByIndex(Index : Integer) : IMoniker;
    function  GetAudioOutDeviceByIndex(Index : Integer) : IMoniker;
    procedure SetVideoDevice(Index : Integer);
    procedure SetAudioInDevice(Index : Integer);
    procedure SetAudioOutDevice(Index : Integer);
    function  GetEnumVideoFormat(Items: TStrings) : Boolean;
    function  GetEnumAudioInFormat(Items: TStrings) : Boolean;
    function  GetEnumAudioOutFormat(Items: TStrings) : Boolean;
    function  GetEnumAudioInput(Items: TStrings) : Integer;
    procedure SetAudioInFormat(Index : Integer);
    procedure SetAudioInLines(Index : Integer);
    procedure EnabledSoundStream(Index, Volume : Integer; Enabled : Boolean);
    function  SetFreq(Value: LongWord): HRESULT;
    function  TunerAutoFreq(Channel : Integer) : Boolean;
    function  TunerStoreFreq : Boolean;
    function  SetTVVideo(Value : Boolean) : Boolean;
    function  SetTVAudio(Value : Boolean) : Boolean;
    function  AvailableTVFormats : TSRVVideoFormats;
    function  TVFormat : TSRVVideoFormat;
    function  TVFormatStr : String;
    procedure RadioTunerBeginScan;
    procedure RadioTunerEndScan;
    procedure PlayDVD(FileName: WideString; Play : Boolean = True);
    function  DVDPlayTitle(TitleIndex : Integer) : Boolean;
    function  DVDPlayChapterInTitle(TitleIndex, ChapterIndex : Integer) : Boolean;
    procedure DVDStepForward;
    procedure DVDRootMenu;
    procedure DVDSavebookmark(FileName: String);
    procedure DVDRestorebookmark(FileName: String);
    procedure DVDTitleMenu;
    function  GetDVDNumberOfChapters(TitleIndex : Integer) : Integer;
    procedure DVDNextChapter;
    procedure DVDPreviousChapter;
    procedure DVDFastForward;
    procedure DVDFastBackward;
    procedure DVDSpeed(Speed : Double);
    procedure DVDMouseMove(X, Y: Integer);
    procedure DVDMouseUp(X, Y: Integer);
    // Sound
    function  GetSoundChannalName(iSoundChannal : Integer;
      var ChannalName : string) : Boolean;
    // Language
    function  GetLanguageName(iLanguage : Integer; var LanguageName : string) : Boolean;
    // Subtitle
    function  IsSubtitlesUsed: Boolean;
    function  GetSubtitleTiming(var delay, speedmul, speeddiv : Integer) : Boolean;
    function  SetSubtitleTiming(delay, speedmul, speeddiv : Integer) : Boolean;
    function  GetSubtitleTextSettings(var lf : LOGFONT; var color : COLORREF; var fShadow, fOutline, fAdvancedRenderer : Bool) : Boolean;
    function  SetSubtitleTextSettings(lf : LOGFONT; color : COLORREF; fShadow, fOutline, fAdvancedRenderer : Bool) : Boolean;
    function  UpdateRegistrySubtitle : Boolean;
    function  GetSubtitlePlacement(var fOverridePlacement : Bool; var xperc, yperc : Integer) : Boolean;
    function  SetSubtitlePlacement(fOverridePlacement : Bool; xperc, yperc : Integer) : Boolean;
    function  GetSubtitleLanguageName(iSubtitleLanguage : Integer;
      var LanguageName : PWideChar) : Boolean;
    procedure ShowDeviceDialog(PropertyPage: TPropertyPage = ppDefault);
    // Close Caption Text
    function  IsCCTextUsed: Boolean;
    function  GetOutputFormatCCText(var lpbmih: TBitmapInfoHeader) : Boolean;
    function  SetOutputFormatCCText(lpbmih: TBitmapInfoHeader) : Boolean;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;

function GetOutputPin(const BF: IBaseFilter): IPin;
function FindAudioSwith(const FG: IFilterGraph): IBaseFilter;
procedure SetAudioStream(const Index: Integer);

    procedure Open;
    procedure Close;
    procedure Play;
    procedure Stop;
    procedure Pause;
    function VideoWidth : Integer;
    function VideoHeight : Integer;
    property State : TRVMPModes read FState;
    property RenderVisible;
    property DisplayRect;
    property SourceRect;
    property DestinationRect;
    property DVDVolumeCount : Integer read GetDVDVolumeCount;
    property DVDCurrentVolume : Integer read GetDVDCurrentVolume;
    property DVDSide : TDVD_Disc_Side read GetDVDSide;
    property DVDTitleCount : Integer read GetDVDTitleCount;
    property VideoSpeed : Double read GetVideoSpeed write SetVideoSpeed;
    property VideoPosition : Double read GetVideoPosition write SetVideoPosition;
    property VideoLength : Double read GetVideoLength;
    property Volume : Integer read GetVolume write SetVolume;
    property Balance : Integer read GetBalance write SetBalance;
    property TunerMode : TSRVTunerModeType read GetTunerMode write SetTunerMode default tmDefault;
    property TunerCountry : Integer read GetTunerCountry write SetTunerCountry default 0;
    property TunerChannel : Integer read GetTunerChannel write SetTunerChannel default 1;
    property TunerType : TTunerInputType read GetTunerType write SetTunerType default TunerInputAntenna;
    property TunerChannelMin : Integer read GetTunerChannelMin;
    property TunerChannelMax : Integer read GetTunerChannelMax;
    property TunerSignalPresent : Boolean read GetTunerSignalPresent;
    property VideoFrequency : Integer read GetVideoFrequency;
    property AudioFrequency : Integer read GetAudioFrequency;
    property Zoom : Integer read FZoom write SetZoom default 1;
    property IndexSoundChannal : Integer read GetIndexSoundChannal write SetIndexSoundChannal default 0;
    property SoundChannalsCount : Integer read GetSoundStreamsCount;
    property IndexSubtitleLanguage : Integer read GetIndexSubtitleLanguage write SetIndexSubtitleLanguage default 0;
    property SubtitleLanguagesCount : Integer read GetSubtitleLanguagesCount;
    property ShowCCText : Boolean read GetShowCCText write SetShowCCText default False;
    property ShowSubtitles : Boolean read GetShowSubtitles write SetShowSubtitles default True;
    property FileNameSubtitles : String read GetFileNameSubtitles write SetFileNameSubtitles;
    property LanguagesCount : Integer read GetLanguagesCount;
    property IndexLanguage: Integer read GetIndexLanguage write SetIndexLanguage;
    property FlipTextSubtitles : Boolean read GetFlipTextSubtitles write SetFlipTextSubtitles;
    property FlipPictureSubtitles : Boolean read GetFlipPictureSubtitles write SetFlipPictureSubtitles;
    property ColorFormatSubtitles : Integer read GetColorFormatSubtitles write SetColorFormatSubtitles;
    property OSDSubtitle : Boolean read GetOSDSubtitle write SetOSDSubtitle;
    property BackgroundColorCCText: TColor read GetBackgroundColorCCText write SetBackgroundColorCCText;
    property BackgroundModeCCText: TBGModeSubtitle read GetBackgroundModeCCText write SetBackgroundModeCCText;
    property ServiceCCText: TCCTextService read GetServiceCCText write SetServiceCCText;
  published
    { Published declarations }
    property Anchors;
    property Align;
    property Hint;
    property Enabled;
    property Color default clBlack;
    property Display;
    property DrawOnPrint;
    property FileName : String read FFileName write FFileName;
    property FullScreen : Boolean read FFullScreen write SetFullScreen default False;
    property GraphMode : TGraphMode read GetGraphMode write SetGraphMode default gmNormal;
    property AutoHideCursor : Integer read FIdleCursor write FIdleCursor;
    property AllSound : Boolean read FAllSound write SetAllSound default False;
    property Height default 200;
    property ShowHint;
    property Visible;
    property TabStop default True;
    property Width default 320;
    property OnClick;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnEnter;
    property OnExit;
    property OnDblClick;
    {$IFDEF RICHVIEWDEF5}
    property OnContextPopup;
    {$ENDIF}
    property OnMouseEnter;
    property OnMouseLeave;
    property OnChange : TSRVMPOnChange read FOnChange write FOnChange;
    property OnBufferVideo : TOnBuffer read FOnBufferVideo write FOnBufferVideo;
    property OnBufferAudio : TOnBuffer read FOnBufferAudio write FOnBufferAudio;
    property OnCursorShowHide : TCursorVisibleEvent read FOnCursorVisible
      write FOnCursorVisible;

    property OnDVDAudioStreamChange : TOnDVDAudioStreamChange read GetOnDVDAudioStreamChange
      write SetOnDVDAudioStreamChange;
    property OnDVDSubpictureStreamChange : TOnDVDSubpictureStreamChange read GetOnDVDSubpictureStreamChange
      write SetOnDVDSubpictureStreamChange;
    property OnDVDCurrentHMSFTime : TOnDVDCurrentHMSFTime read FOnDVDCurrentHMSFTime
      write FOnDVDCurrentHMSFTime;
    property OnDVDTitleChange : TOnDVDTitleChange read GetOnDVDTitleChange
      write SetOnDVDTitleChange;
    property OnDVDChapterStart : TOnDVDChapterStart read GetOnDVDChapterStart
      write SetOnDVDChapterStart;
    property OnDVDAngleChangee : TOnDVDChange read GetOnDVDAngleChange
      write SetOnDVDAngleChange;
    property OnDVDButtonChange : TOnDVDChange read GetOnDVDButtonChange
      write SetOnDVDButtonChange;
    property OnDVDPlaybackRateChange : TOnDVDPlaybackRateChange
      read GetOnDVDPlaybackRateChange write SetOnDVDPlaybackRateChange;
    property OnDVDCurrentTime : TOnDVDCurrentTime
      read GetOnDVDCurrentTime write SetOnDVDCurrentTime;
    property OnDVDKaraokeMode : TOnDVDKaraokeMode read GetOnDVDKaraokeMode
      write SetOnDVDKaraokeMode;
  end;

procedure Register;

implementation
uses Math;

procedure Register;
begin
  RegisterComponents('SRichView', [TSRVMediaPlayer]);
end;

constructor TSRVMediaPlayer.Create(AOwner: TComponent);
begin
  inherited;
    MyVideoWindow := nil;
    MyMediaPosition := nil;
    MyBasicVideo := nil;
    FFullScreen := False;

    IndexInputLines := -1;
    IndexAudioInFormat := -1;
    InputLines := TStringList.Create;
    FDVDSpeed := 1;
    FZoom := 1;
    FIndexSoundChannal := 0;
    FIndexSubtitleLanguage := 0;
    FAllSound := False;
    DefaultLeft := 1;
    DefaultTop := 1;
    DefaultWidth := 1;
    DefaultHeight := 1;

    FState := rvmpNotReady;
    CoInitialize(nil);

    VideoMediaTypes := TEnumMediaType.Create;
    AudioMediaInTypes := TEnumMediaType.Create;
    AudioMediaOutTypes := TEnumMediaType.Create;

//    AudioDevice := TSysDevEnum.Create(CLSID_CWaveinClassManager);
//    AudioDevice:= TSysDevEnum.Create(CLSID_AudioRendererCategory);
//    AudioDevice:= TSysDevEnum.Create(IID_IAMCameraControl);
    AudioInDevice:= TSysDevEnum.Create(CLSID_AudioInputDeviceCategory);
    AudioOutDevice:= TSysDevEnum.Create(CLSID_CWaveOutClassManager);

    Filter := TFilter.Create(nil);
    AudioInFilter := TFilter.Create(nil);
    AudioOutFilter := TFilter.Create(nil);
    SampleGrabberVideo := TSampleGrabber.Create(nil);
    SampleGrabberAudio := TSampleGrabber.Create(nil);
    FilterGraph := TFilterGraph.Create(nil);
    FilterGraph.Mode := gmNormal;
    FilterGraph.OnDVDCurrentHMSFTime := HookDVDCurrentHMSFTime;

    Filter.FilterGraph := FilterGraph;
    AudioInFilter.FilterGraph := FilterGraph;
    AudioOutFilter.FilterGraph := FilterGraph;
    SampleGrabberVideo.FilterGraph := FilterGraph;

    SampleGrabberAudio.FilterGraph := FilterGraph;
    SampleGrabberAudio.MediaType.Free;
    SampleGrabberAudio.MediaType := TMediaType.Create(MEDIATYPE_Audio);
    SampleGrabberAudio.MediaType.SubType := MEDIASUBTYPE_PCM;
    SampleGrabberAudio.MediaType.FormatType := FORMAT_WaveFormatEx;
    SampleGrabberAudio.UpdateMediaType;

    Application.OnIdle := MyIdleHandler;
end;

destructor TSRVMediaPlayer.Destroy;
begin
  Close;
//    if FState <> rvmpNotReady then
//      CoUninitialize;

  Filter.Destroy;
  AudioInFilter.Destroy;
  AudioOutFilter.Destroy;
  SampleGrabberVideo.Destroy;
  SampleGrabberAudio.Destroy;
  FilterGraph.Destroy;

  AudioInDevice.Destroy;
  AudioOutDevice.Destroy;

  VideoMediaTypes.Destroy;
  AudioMediaInTypes.Destroy;
  AudioMediaOutTypes.Destroy;

  inherited;
end;

procedure TSRVMediaPlayer.Loaded;
begin
  inherited;
    FDisplayRect := Rect(0, 0, Width, Height);
end;

function  TSRVMediaPlayer.GetGraphMode : TGraphMode;
begin
  Result := FilterGraph.Mode;
end;

procedure TSRVMediaPlayer.SetGraphMode(Value : TGraphMode);
begin
  Close;
  FilterGraph.Mode := Value;
end;

function  TSRVMediaPlayer.GetTunerMode : TSRVTunerModeType;
var
     Value : Cardinal;
     hr : HRESULT;
begin
  Result := tmDefault;
  If Assigned(MyAMTVTuner) then begin
    hr := MyAMTVTuner.get_Mode(Value);
    If Succeeded(hr) then
      case Value of
        AMTUNER_MODE_TV : Result := tmTV;
        AMTUNER_MODE_FM_RADIO : Result := tmFMRadio;
        AMTUNER_MODE_AM_RADIO : Result := tmAMRadio;
        AMTUNER_MODE_DSS : Result := tmDSS;
      end;
  end;
end;

procedure TSRVMediaPlayer.SetTunerMode(Value : TSRVTunerModeType);
var
     lModes : Integer;
     nValue : Cardinal;
     hr : HRESULT;
begin
  If not Assigned(MyAMTVTuner) then
    exit;
  case Value of
    tmTV : nValue := AMTUNER_MODE_TV;
    tmFMRadio : nValue := AMTUNER_MODE_FM_RADIO;
    tmAMRadio : nValue := AMTUNER_MODE_AM_RADIO;
    tmDSS : nValue := AMTUNER_MODE_DSS;
  else
    nValue := AMTUNER_MODE_DEFAULT;
  end;

  // Check whether the mode is supported.
  lModes := 0;
  hr := MyAMTVTuner.GetAvailableModes(lModes);
  if Succeeded(hr) and (lModes and nValue <> 0) then
    // Set the mode.
    MyAMTVTuner.put_Mode(nValue);
end;

function  TSRVMediaPlayer.GetTunerCountry : Integer;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.get_CountryCode(Result)
  else
    Result := 0;
end;

procedure TSRVMediaPlayer.SetTunerCountry(Value : Integer);
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.put_CountryCode(Value);
end;

function  TSRVMediaPlayer.GetTunerType : TTunerInputType;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.get_InputType(0, Result)
  else
    Result := TunerInputAntenna;
end;

procedure TSRVMediaPlayer.SetTunerType(Value : TTunerInputType);
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.put_InputType(0, Value)
end;

function  TSRVMediaPlayer.GetTunerChannel : Integer;
var
     VSubChannel, ASubChannel : Integer;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.get_Channel(Result, VSubChannel, ASubChannel)
  else
    Result := 0;
end;

procedure TSRVMediaPlayer.SetTunerChannel(Value : Integer);
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.put_Channel(Value, AMTUNER_SUBCHAN_DEFAULT, AMTUNER_SUBCHAN_DEFAULT);
end;

function  TSRVMediaPlayer.GetTunerChannelMin : Integer;
var
     Value : Integer;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.ChannelMinMax(Result, Value)
  else
    Result := 0;
end;

function  TSRVMediaPlayer.GetTunerChannelMax : Integer;
var
     Value : Integer;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.ChannelMinMax(Value, Result)
  else
    Result := 0;
end;

function  TSRVMediaPlayer.GetTunerSignalPresent : Boolean;
var
     SignalStrength: Integer;
     hr : HRESULT;
begin
  Result := False;
  If Assigned(MyAMTVTuner) then begin
    hr := MyAMTVTuner.SignalPresent(SignalStrength);
    If Succeeded(hr) then
      case SignalStrength of
        AMTUNER_HASNOSIGNALSTRENGTH: Result := False;
        AMTUNER_NOSIGNAL           : Result := False;
        AMTUNER_SIGNALPRESENT      : Result := True;
      else
        Result := True;
      end;
  end;
end;

function  TSRVMediaPlayer.GetVideoFrequency : Integer;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.get_VideoFrequency(Result)
  else
    Result := 0;
end;

function  TSRVMediaPlayer.GetAudioFrequency : Integer;
begin
  If Assigned(MyAMTVTuner) then
    MyAMTVTuner.get_AudioFrequency(Result)
  else
    Result := 0;
end;

function  TSRVMediaPlayer.TunerAutoFreq(Channel : Integer) : Boolean;
var
     Signal : Integer;
     hr: HRESULT;
begin
  Result := False;
  If Assigned(MyAMTVTuner) then begin
    hr := MyAMTVTuner.AutoTune(Channel, Signal);
    Result := Succeeded(hr) and (Signal <> 0);
  end;
end;

function  TSRVMediaPlayer.TunerStoreFreq : Boolean;
var
     hr: HRESULT;
begin
  Result := False;
  If Assigned(MyAMTVTuner) then begin
    hr := MyAMTVTuner.StoreAutoTune();
    Result := Succeeded(hr);
  end;
end;

function  TSRVMediaPlayer.GetVideoSpeed : Double;
begin
  Result := -1;
  case GraphMode of
  gmNormal :
    If Assigned(MyMediaPosition) then
      MyMediaPosition.get_Rate(Result);
  gmDVD:
    Result := FDVDSpeed;
  end;
end;

procedure TSRVMediaPlayer.SetVideoSpeed(Value : Double);
begin
  case GraphMode of
  gmNormal :
    If Assigned(MyMediaPosition) then
      MyMediaPosition.put_Rate(Value);
  gmDVD:
    DVDSpeed(Value);
  end;
end;

function  TSRVMediaPlayer.GetVideoPosition : Double;
begin
  If FilterGraph.Active and (GraphMode = gmDVD) then
    Result := DVDCurrentPos
  else If Assigned(MyMediaPosition) then
    MyMediaPosition.get_CurrentPosition(Result)
  else
    Result := -1;
end;

procedure TSRVMediaPlayer.SetVideoPosition(Value : Double);

  procedure DVD_SetTrackTime(NewValue : Cardinal);
  var
       DvdControl2: IDvdControl2;
       DvdCmd: IDvdCmd;
//       HMSFStart: TDVD_HMSF_TimeCode;
       HMSFTime: TDVD_HMSF_TimeCode;
  begin
    if Succeeded(FilterGraph.QueryInterface(IDvdControl2, DvdControl2)) then begin
      //MiliSecToHours(0, HMSFStart.bHours, HMSFStart.bMinutes, HMSFStart.bSeconds);
      MiliSecToHours(NewValue, HMSFTime.bHours, HMSFTime.bMinutes, HMSFTime.bSeconds);
      //DvdControl2.PlayAtTimeInTitle(dvdTitelNr, HMSFTime, DVD_CMD_FLAG_Flush, DvdCmd);
      //DvdControl2.PlayPeriodInTitleAutoStop(dvdTitelNr,HMSFStart,HMSFTime,DVD_CMD_FLAG_Flush,DvdCmd);
      DvdControl2.PlayAtTime(@HMSFTime, DVD_CMD_FLAG_Flush {DVD_CMD_FLAG_None}, DvdCmd);
      DVDCurrentPos := NewValue;
      DvdCmd := nil;
      DVDControl2 := nil;
    end;
  end;

begin
  if FilterGraph.Active and (GraphMode = gmDVD) then
    DVD_SetTrackTime(Round(Value))
  else If Assigned(MyMediaPosition) then
    MyMediaPosition.put_CurrentPosition(Value);
end;

function  TSRVMediaPlayer.GetVideoLength : Double;

  function DVD_GetTrackTotalLength : Double;
  var
       DVDInfo2: IDVDInfo2;
       HMSFTimeCode: TDVD_HMSF_TimeCode;
       x : ULONG;
  begin
    if Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
      DVDInfo2.GetTotalTitleTime(HMSFTimeCode, x);
      DVDInfo2 := nil;

      with HMSFTimeCode do
        Result := HoursToMiliSec(bHours, bMinutes, bSeconds);
    end else Result := 0;
  end;

begin
  If FilterGraph.Active and (GraphMode = gmDVD) then
    Result := DVD_GetTrackTotalLength
  else If Assigned(MyMediaPosition) then
    MyMediaPosition.get_Duration(Result)
  else
    Result := -1;
end;

procedure TSRVMediaPlayer.Open;
var
//  multiplexer: IBaseFilter;
//  Writer: IFileSinkFilter;
     PinList: TPinList;
     i: integer;
     hr : HRESULT;
begin
  {calling RenderFile - filter graph is built automatically}
  if not FileExists(FileName) and (GraphMode = gmNormal) then exit;
  Close;

  SampleGrabberVideo.OnBuffer := FOnBufferVideo;
  SampleGrabberAudio.OnBuffer := FOnBufferAudio;
  // ,        
{  hr := FilterGraph.RenderStream(&PIN_CATEGORY_VBI, 0, pCap, 0, 0);
  if Failed(hr) then
    hr := pBuild->RenderStream(&PIN_CATEGORY_CC, 0, pCap, 0, 0);}

  try
  FilterGraph.Active := True;
  FilterGraph.ClearGraph;
  DVDCurrentPos := 0;
  case GraphMode of
  gmNormal :
    begin
      FilterGraph.RenderFile(FileName);
    end;
  gmCapture:
    begin
      with FilterGraph.CaptureGraph do begin
        if Filter.BaseFilter.DataLength > 0 then
          RenderStream(@PIN_CATEGORY_PREVIEW, nil, Filter.Filter, SampleGrabberVideo.BaseFilter, nil);

        // set the output filename
//        SetOutputFileName(MEDIASUBTYPE_Avi, 'c:\1.avi', multiplexer, Writer);

        If AudioInFilter.BaseFilter.DataLength > 0 then
          RenderStream(@PIN_CATEGORY_PREVIEW, nil, AudioInFilter.Filter, SampleGrabberAudio.BaseFilter, AudioOutFilter.Filter);
      end;
    end;
  gmDVD:
    begin
      FilterGraph.Active := False;
      PlayDVD(FileName, False);
    end;
  end;

{ FilterGraph.FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21);
 FilterGraph.FilterGraph.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);
 If FilterGraph.FilterGraph.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect) = S_OK then
         Begin
           // Reset to first stream if there is more than one stream
           If MyAMStreamSelect.Count(DWord(I)) = S_OK then
             If I > 1 then MyAMStreamSelect.Enable(0,AMSTREAMSELECTENABLE_ENABLE);
           MyAMStreamSelect:= nil;
         End;}

  if AudioInFilter.BaseFilter.DataLength > 0 then
  begin
    PinList := TPinList.Create(AudioInFilter.Filter);
    i := 0;
    while i < PinList.Count do
      if PinList.PinInfo[i].dir = PINDIR_OUTPUT then
        begin
          if IndexAudioInFormat <> -1 then
            with (PinList.Items[i] as IAMStreamConfig) do
              SetFormat(AudioMediaInTypes.Items[IndexAudioInFormat].AMMediaType^);
          PinList.Delete(i);
        end else inc(i);
    if IndexInputLines <> -1 then
      with (PinList.Items[IndexInputLines] as IAMAudioInputMixer) do
        put_Enable(true);
    PinList.Free;
  end;

  {acquiring ImediaControl interface}
  FilterGraph.QueryInterface(IID_IMediaControl, MyMediaControl);
  {note - MyMediaControl: IMediaControl}

  {acquiring IID_IMediaSeeking interface}
  FilterGraph.QueryInterface(IID_IMediaSeeking, MyMediaSeeking);
  {note - MyMediaSeeking: IMediaSeeking}

  {acquiring IID_IMediaPosition interface}
  FilterGraph.QueryInterface(IID_IMediaPosition, MyMediaPosition);
  {note - MediaPosition: IMediaPosition}

  {acquiring IID_IBasicVideo interface}
  FilterGraph.QueryInterface(IID_IBasicVideo, MyBasicVideo);
  {note - BasicVideo: IBasicVideo}

  {acquiring IID_IBasicAudio interface}
  FilterGraph.QueryInterface(IID_IBasicAudio, MyBasicAudio);
  {note - MyBasicAudio: IBasicAudio}

  {acquiring IVideoWindow interface}
  FilterGraph.QueryInterface(IID_IVideoWindow, MyVideoWindow);
  {note - VideoWindow: IVideoWindow}

  {acquiring IMediaEvent interface}
  FilterGraph.QueryInterface(IID_IMediaEvent, MyMediaEvent);
  {note - MediaEvent: IMediaEvent}

  {acquiring  IMediaEventEx interface}
  FilterGraph.QueryInterface(IID_IMediaEventEx, MyMediaEventEx);
  {note - MediaEventEx: IMediaEventEx}

  FilterGraph.QueryInterface(IID_IBaseFilter, MyBaseFilter);
  FilterGraph.QueryInterface(IID_IFilterGraph, MyFilterGraph);

  FilterGraph.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);
  FilterGraph.QueryInterface(IAMWstDecoder, MyIAMWstDecoder);

{  FilterGraph.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);
  SampleGrabberAudio.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);
  SampleGrabberAudio.SampleGrabber.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);
  SampleGrabberAudio.BaseFilter.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);

  //  FilterDevice:= TSysDevEnum.Create(CLSID_AudioInputDeviceCategory);
{    SampleGrabberAudio.FilterGraph := FilterGraph;
    SampleGrabberAudio.MediaType.Free;
    SampleGrabberAudio.MediaType := TMediaType.Create(MEDIATYPE_Text);
    SampleGrabberAudio.MediaType.SubType := MEDIASUBTYPE_TELETEXT;
    SampleGrabberAudio.MediaType.FormatType := FORMAT_None;
    SampleGrabberAudio.UpdateMediaType;
    SampleGrabberAudio.QueryInterface(IID_IAMStreamSelect, MyAMStreamSelect);
    SampleGrabberAudio.QueryInterface(IID_IAMLine21Decoder, Line21);}

  If FilterGraph.CaptureGraph <> nil then begin
    FilterGraph.CaptureGraph.QueryInterface(IAMWstDecoder, MyIAMWstDecoder);

    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
      nil, Filter.Filter, IAMWstDecoder, MyIAMWstDecoder);

    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
      nil, Filter.Filter, IID_IAMStreamSelect, MyAMStreamSelect);

    FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21);

    hr := FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
      nil, Filter.Filter, IID_IAMLine21Decoder, Line21);
    if Succeeded(hr) then begin
      Line21.SetServiceState(AM_L21_CCSTATE_Off);
      // (Use AM_L21_CCSTATE_On to enable.)
      Line21._Release();
    end;
  end;

  // TV tuner
  If GraphMode = gmCapture then begin
    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
              nil, Filter.Filter, IID_IAMTuner, MyAMTuner);
    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
       	      nil, Filter.Filter, IID_IAMTVTuner, MyAMTVTuner);
    If Assigned(MyAMTVTuner) then begin
      MyAMTVTuner.put_Mode(AMTUNER_MODE_TV);
//      MyAMTVTuner.put_CountryCode(7);
      MyAMTVTuner.put_ConnectInput(0);
      MyAMTVTuner.put_InputType(0, TunerInputCable);
    end;

    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
	      nil, Filter.Filter, IID_IAMCrossbar, MyAMCrossbar);
    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
              nil, Filter.Filter, IID_IAMVideoControl, MyAMVideoControl);
    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
              nil, Filter.Filter, IID_IAMVideoProcAmp, MyAMVideoProcAmp);
    FilterGraph.CaptureGraph.FindInterface(@LOOK_UPSTREAM_ONLY,
              nil, Filter.Filter, IID_IAMTVAudio, MyAMTVAudio);
  end;

  {placing video window on the panel}
  if FDisplay = nil then begin
    MyVideoWindow.put_Owner(Handle);
//    MyVideoWindow.put_MessageDrain(Handle);
  end else begin
    MyVideoWindow.put_Owner(FDisplay.Handle);
//    MyVideoWindow.put_MessageDrain(FDisplay.Handle);
  end;

  MyVideoWindow.put_MessageDrain(Handle);

  MyVideoWindow.put_WindowStyle(WS_CHILD OR WS_CLIPSIBLINGS);
  MyVideoWindow.SetWindowPosition(FDisplayRect.Left, FDisplayRect.Top,
                                  FDisplayRect.Right - FDisplayRect.Left,
                                  FDisplayRect.Bottom - FDisplayRect.Top);

  MyBasicVideo.SetDefaultSourcePosition;
  MyBasicVideo.get_SourceLeft(DefaultLeft);
  MyBasicVideo.get_SourceTop(DefaultTop);
  MyBasicVideo.get_SourceWidth(DefaultWidth);
  MyBasicVideo.get_SourceHeight(DefaultHeight);
  GetVolume;

  FState := rvmpOpen;
  except
    Close;
  end;
end;

procedure TSRVMediaPlayer.Play;
begin
  {playing video}
  if (FState = rvmpNotReady) then exit;

  if (GraphMode <> gmCapture) and (GraphMode <> gmDVD) and HandleAllocated then begin
    if (SetTimer(Handle, 1, 1000, nil) = 0) then
      raise EOutOfResources.Create('Out of resource for installing timer');
  end;

  if (GraphMode = gmDVD) then PlayDVD('')
  else FilterGraph.Play;

//  MyMediaControl.Run;

  FState := rvmpPlaying;
end;

procedure TSRVMediaPlayer.Stop;
begin
  if (FState = rvmpNotReady) then exit;
  if HandleAllocated then
    KillTimer(Handle, 1);
//  MyMediaControl.Stop;
  FilterGraph.Stop;
  FState := rvmpStopped;
end;

procedure TSRVMediaPlayer.Pause;
begin
  if (FState = rvmpNotReady) then exit;
  if HandleAllocated then
    KillTimer(Handle, 1);
//  MyMediaControl.Pause;
  FilterGraph.Pause;
  FState := rvmpPaused;
end;

procedure TSRVMediaPlayer.Close;
begin
  if (State = rvmpPlaying) or (State = rvmpPaused) then
//    MyMediaControl.Stop;
    FilterGraph.Stop;
  if HandleAllocated then
    KillTimer(Handle, 1);
//  CoUninitialize;
  FilterGraph.Active := False;
  FState := rvmpNotReady;
end;

function TSRVMediaPlayer.GetRenderVisible: Boolean;
var
     v : LongBool;
begin
  if MyVideoWindow = nil then v := False
  else MyVideoWindow.get_Visible(v);
  Result := v;
end;

procedure TSRVMediaPlayer.SetRenderVisible(Value : Boolean);
begin
  if MyVideoWindow = nil then exit;
  MyVideoWindow.put_Visible(Value);
end;

procedure TSRVMediaPlayer.SetRotation(Value : TRVDocRotation);
begin
  case Rotation of
  rvrot90:
    begin
      with SourceRect do
        FRotationSourceRect := Rect(Top, Right, Bottom, Left);
    end;
  rvrot180:
    begin
      with SourceRect do
        FRotationSourceRect := Rect(Right, Bottom, Left, Top);
    end;
  rvrot270:
    begin
      with SourceRect do
        FRotationSourceRect := Rect(Bottom, Left, Top, Right);
    end;
  else
    FRotationSourceRect := SourceRect;
  end;

  inherited;
end;

procedure TSRVMediaPlayer.SetDisplay(Value : TWinControl);
begin
  inherited;
  if MyVideoWindow <> nil then
    if Value = nil then begin
      MyVideoWindow.put_Owner(Handle);
//      MyVideoWindow.put_MessageDrain(Handle);
    end else begin
      MyVideoWindow.put_Owner(Value.Handle);
//      MyVideoWindow.put_MessageDrain(Value.Handle);
    end;
end;

procedure TSRVMediaPlayer.SetDisplayRect(Value : TRect);
begin
  inherited;
  if MyVideoWindow <> nil then
    with Value do
      if Rotation = rvrotNone then
        MyVideoWindow.SetWindowPosition(Left, Top, Right - Left, Bottom - Top)
      else
        MyVideoWindow.SetWindowPosition(0, 0, 0, 0)
end;

procedure TSRVMediaPlayer.SetSourceRect(Value : TRect);
begin
  inherited;

  if Rotation <> rvrotNone then
    FSourceRect := Rect(0, 0, 0, 0);
  SetRotation(Rotation);
  if MyBasicVideo <> nil then
    with FRotationSourceRect do
      MyBasicVideo.SetSourcePosition(Left, Top, Right - Left, Bottom - Top);
end;

procedure TSRVMediaPlayer.SetDestinationRect(Value : TRect);
begin
  inherited;
  if MyBasicVideo <> nil then
    MyBasicVideo.SetDestinationPosition(Value.Left, Value.Top,
                                        Value.Right - Value.Left,
                                        Value.Bottom - Value.Top);
end;

function TSRVMediaPlayer.VideoWidth : Integer;
begin
  Result := 0;
  if MyBasicVideo <> nil then
    MyBasicVideo.get_VideoWidth(Result);
end;

function TSRVMediaPlayer.VideoHeight : Integer;
begin
  Result := 0;
  if MyBasicVideo <> nil then
    MyBasicVideo.get_VideoHeight(Result);
end;

function  TSRVMediaPlayer.GetVolume : Integer;
begin
  Result := 0;
  If not Assigned(MyBasicAudio) then exit;

  If FIndexSoundChannal = -1 then
    MyBasicAudio.get_Volume(Result)
  else
    Result := GetVolumeSoundStream(FIndexSoundChannal);

  FVolume := Result;
  If Result <> - 10000 then
    Result := Round(Exp((10000 + Result) * Ln(10) / 5000))
  else Result := 0;
end;

procedure TSRVMediaPlayer.SetVolume(Value : Integer);
begin
  Value := Min(Max(Value, 0), 100);
  //   
  If Value <> 0 then
    Value := Round(5000 * Ln(Value) / Ln(10));
  Value := Value - 10000;
  FVolume := Value;
  If FIndexSoundChannal = -1 then
    MyBasicAudio.put_Volume(Value)
  else
    SetSoundStream(FIndexSoundChannal, Value);
end;

function  TSRVMediaPlayer.GetBalance : Integer;
begin
  MyBasicAudio.get_Balance(Result);
  Result := 100 - (Result div 100);
end;

procedure TSRVMediaPlayer.SetBalance(Value : Integer);
begin
  Value := Min(Max(Value, 0), 100);
  Value := (Value - 100) * 100;
  MyBasicAudio.put_Balance(Value);
end;

procedure TSRVMediaPlayer.MediaPlayerNotify(var Msg : TMessage);
var
     evCode, param1, param2 : integer;
begin
  inherited;

  while Succeeded(MyMediaEventEx.GetEvent(evCode, param1, param2, 0)) do begin
    MyMediaEventEx.FreeEventParams(evCode, param1, param2);
    case evCode of
      EC_QUALITY_CHANGE :
        begin
           If Assigned(OnClick) then
             OnClick(self);
        end;
      EC_CLOCK_CHANGED :
        begin
           If Assigned(OnClick) then
             OnClick(self);
        end;
      EC_TIME :
        begin
           If Assigned(OnClick) then
             OnClick(self);
        end;
      EC_NOTIFY_WINDOW :
        begin
           If Assigned(OnClick) then
             OnClick(self);
        end;
      //  
      EC_COMPLETE :
        begin
          if HandleAllocated then
            KillTimer(Handle, 1);
           If Assigned(OnClick) then
             OnClick(self);
        end;
      // 
      EC_REPAINT :
        begin
           If Assigned(OnClick) then
             OnClick(self);
        end;
      //   
      EC_VIDEO_SIZE_CHANGED :
        begin
        end;
      EC_OPENING_FILE :
        begin
        end;
      EC_FULLSCREEN_LOST :
        begin
        end;
      EC_NEED_RESTART :
        begin
           If Assigned(OnClick) then
             OnClick(self);
        end;
      EC_DISPLAY_CHANGED :
        begin
        end;
    end;
  end;
end;

procedure TSRVMediaPlayer.SetFullScreen(Value : Boolean);
begin
  FFullScreen := Value;
  If Value then begin
    MyVideoWindow.put_Owner(0);
    MyVideoWindow.put_FullScreenMode(LongBool(-1));
    MyVideoWindow.SetWindowPosition(Screen.DesktopLeft, Screen.DesktopTop,
                                    Screen.DesktopWidth,
                                    Screen.DesktopHeight);
  end else begin
    MyVideoWindow.put_Owner(Display.Handle);
    MyVideoWindow.put_FullScreenMode(LongBool(0));
    MyVideoWindow.SetWindowPosition(FDisplayRect.Left, FDisplayRect.Top,
                                    FDisplayRect.Right - FDisplayRect.Left,
                                    FDisplayRect.Bottom - FDisplayRect.Top);
  end;
end;

procedure TSRVMediaPlayer.GetCurrentImageToStream(Stream : TStream);
var
     b : TBitmap;
begin
  b := GetCurrentImage;
  b.SaveToStream(Stream);
  b.Destroy;
end;

{------------------------------------------------------------------------------
  Params  : <Dib>     Pointer to DIB 
  Returns : <Result>  Size of header 

  Descript: Get header size of DIB 
  Notes   : 
 ------------------------------------------------------------------------------} 

function GetDibHeaderSize(Dib: PBitmapInfo): Integer; 
begin 
  // Defaults for colors used, so get the color table count from 
  // the number of bits per pixel 
  if (Dib.bmiHeader.biClrUsed = 0) then 
  begin 
    case Dib.bmiHeader.biBitCount of 
      1: Dib.bmiHeader.biClrUsed := 2; 
      2: Dib.bmiHeader.biClrUsed := 4; 
      3: Dib.bmiHeader.biClrUsed := 8; 
      4: Dib.bmiHeader.biClrUsed := 16; 
      5: Dib.bmiHeader.biClrUsed := 32; 
      6: Dib.bmiHeader.biClrUsed := 64; 
      7: Dib.bmiHeader.biClrUsed := 128; 
      8: Dib.bmiHeader.biClrUsed := 256; 
    end; 
  end; 
  Result := Dib.bmiHeader.biSize + (Dib.bmiHeader.biClrUsed * 4); 
  if ((Dib.bmiHeader.biCompression = BI_BITFIELDS) and (Dib.bmiHeader.biSize = SizeOf(TBitmapInfoHeader))) then 
    Result := Result + 12; 
end; 

{------------------------------------------------------------------------------ 
  Params  : <Width>   Width in pixels 
            <Depth>   Depth in pixels 
  Returns : <Result>  Size of header 

  Descript: Return 4 byte aligned data 
  Notes   : 
 ------------------------------------------------------------------------------}

function AlignScan(Width: Dword; Depth: Dword): Dword; 
begin 
  Result := (((Width * Depth) + $1F) and $FFFFFFE0) div 8; 
end; 

{------------------------------------------------------------------------------ 
  Params  : <Dib>     Pointer to DIB 
  Returns : <Result>  Size of data 

  Descript: Return DIB data size 
  Notes   : 
 ------------------------------------------------------------------------------} 

function GetDibDataSize(Dib: PBitmapInfo): Integer; 
begin 
  case Dib.bmiHeader.biCompression of 
    BI_RGB, BI_BITFIELDS: 
      begin 
        Result := Integer(AlignScan(Dword(Dib.bmiHeader.biWidth), Dword(Dib.bmiHeader.biBitCount))) * Abs(Dib.bmiHeader.biHeight); 
        Dib.bmiHeader.biSizeImage := Result; 
      end; 
    else Result := Dib.bmiHeader.biSizeImage; 
  end; 
end; 

{------------------------------------------------------------------------------ 
  Params  : <Dib>     Pointer to DIB 
  Returns : <Result>  Handle to bitmap (0 if none) 

  Descript: Convert packed DIB into DIB section (handle) 
  Notes   : The caller should release the handle (e.g. with DeleteObject(x)! 
 ------------------------------------------------------------------------------} 

function PackedDibToDibSection(Dib: PBitmapInfo): HBITMAP; 
var 
  BmHead   : TBitmapInfoHeader;
  HeadSize : Integer; 
  DataSize : Integer; 
  DataBits : Pointer; 
begin 
  CopyMemory(@BmHead, Dib, SizeOf(Bmhead));
  // Get header and data size
  HeadSize := GetDibHeaderSize(@BmHead);
  DataSize := GetDibDataSize(@BmHead);
  // Create DibSection based on header
  Result := CreateDibSection(0, Dib^, DIB_RGB_COLORS, DataBits, 0, 0);
  // Copy bitmap to it
  if Result <> 0 then
    CopyMemory(DataBits, Pointer(@PByteArray(Dib)^[HeadSize]), DataSize); 
end;

{------------------------------------------------------------------------------ 
  Params  : - 
  Returns : <Result>  Handle to bitmap (0 if none) 

  Descript: Get current image 
  Notes   : The caller must release the handle with DeleteObject(x)! 
 ------------------------------------------------------------------------------}

function  TSRVMediaPlayer.DVDGetImage: HBITMAP;
var
  Local       : PBitmapInfo;
  FilterState : TFilter_State;
  Hr          : HRESULT;
  ASize  : Integer;

begin
  Result := 0;
  try
    if Assigned(MyBasicVideo) then
    begin
      MyMediaControl.Pause;
      MyMediaControl.GetState(1, FilterState);
      // Check required memory
      ASize := 0;
      Local := nil;
      Hr := MyBasicVideo.GetCurrentImage(ASize, Local^);
      if Succeeded(Hr) then
      begin
        GetMem(Local, ASize);
        try
          Hr := MyBasicVideo.GetCurrentImage(ASize, Local^);
          if Succeeded(Hr) then
            Result := PackedDibToDibSection(Local);
        finally
          FreeMem(Local);
        end;
      end;
      MyMediaControl.Run;
    end;
  except
    ShowMessage('DirectShow GetCurrentImage error');
    Result := 0;
  end;
end;

function TSRVMediaPlayer.GetCurrentImage : TBitmap;
begin
  Result := TBitmap.Create;
  Result.Width := VideoWidth;
  Result.Height := VideoHeight;

  If (GraphMode = gmDVD) then begin
    Result.Handle := DVDGetImage
  end else
    SampleGrabberVideo.GetBitmap(Result);
end;

procedure TSRVMediaPlayer.GetCurrentImageToFile(FileName : String);
var
     b : TBitmap;
begin
  b := GetCurrentImage;
  b.SaveToFile(FileName);
  b.Destroy;
end;

procedure TSRVMediaPlayer.WndProc(var Message: TMessage);
begin
  inherited;

  case Message.Msg of
    WM_TIMER:
      If Message.WParam = 1 then
        Timer;
  end;
end;

procedure TSRVMediaPlayer.Timer;
var
     plen, pllPos  : Double;
begin
  If (GraphMode = gmDVD) or not Assigned(MyMediaPosition) or
     (State <> rvmpPlaying) then exit;

  MyMediaPosition.get_Duration(plen);
  MyMediaPosition.get_CurrentPosition(pllPos);

  If Assigned(OnChange) then
    OnChange(Self, pllPos, plen);
end;

procedure TSRVMediaPlayer.HookDVDCurrentHMSFTime(Sender: TObject;
  HMSFTimeCode: TDVD_HMSF_TimeCode; TimeCode: TDVD_TimeCode);
var
     DVDMaxPos : Integer;
     hmsftc : TDVD_HMSF_TimeCode;
     x : ULONG;
     DVDInfo2 : IDVDInfo2;
begin
  If Assigned(FOnDVDCurrentHMSFTime) then
    FOnDVDCurrentHMSFTime(Sender, HMSFTimeCode, TimeCode);

  // DVD loaded
  if Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    with HMSFTimeCode do
      DVDCurrentPos := HoursToMiliSec(bHours, bMinutes, bSeconds);

    DVDInfo2.GetTotalTitleTime(hmsftc, x);
    DVDInfo2 := nil;

    with hmsftc do
      DVDMaxPos := HoursToMiliSec(bHours, bMinutes, bSeconds);

    If Assigned(OnChange) then
      OnChange(Self, DVDCurrentPos, DVDMaxPos);
  end;
end;

function TSRVMediaPlayer.GetEnumVideoDevice(Items: TStrings) : Boolean;
var
     i: integer;
     guid: TGUID;
begin
  Result := True;

  guid := CLSID_VideoInputDeviceCategory;
  case GraphMode of
    gmCapture:
      guid := CLSID_VideoInputDeviceCategory;
    gmDVD:
      guid := CLSID_VideoInputDeviceCategory;
  end;

  Items.Clear;
  VideoDevice := TSysDevEnum.Create(guid);
  if VideoDevice.CountFilters > 0 then
    for i := 0 to VideoDevice.CountFilters - 1 do
      Items.Add(String(VideoDevice.Filters[i].FriendlyName));
end;

function TSRVMediaPlayer.GetEnumAudioInDevice(Items: TStrings) : Boolean;
var
     i: integer;
begin
  Result := True;

  Items.Clear;
  if AudioInDevice.CountFilters > 0 then
    for i := 0 to AudioInDevice.CountFilters - 1 do
      Items.Add(String(AudioInDevice.Filters[i].FriendlyName));
end;

function TSRVMediaPlayer.GetEnumAudioOutDevice(Items: TStrings) : Boolean;
var
     i: integer;
begin
  Result := True;

  Items.Clear;
  if AudioOutDevice.CountFilters > 0 then
    for i := 0 to AudioOutDevice.CountFilters - 1 do
      Items.Add(String(AudioOutDevice.Filters[i].FriendlyName));
end;

function TSRVMediaPlayer.GetVideoDeviceByIndex(Index : Integer) : IMoniker;
begin
  Result := VideoDevice.GetMoniker(Index);
end;

function TSRVMediaPlayer.GetAudioInDeviceByIndex(Index : Integer) : IMoniker;
begin
  Result := AudioInDevice.GetMoniker(Index);
end;

function TSRVMediaPlayer.GetAudioOutDeviceByIndex(Index : Integer) : IMoniker;
begin
  Result := AudioOutDevice.GetMoniker(Index);
end;

procedure TSRVMediaPlayer.SetVideoDevice(Index : Integer);
var
  PinList: TPinList;
  i: integer;
begin
  FilterGraph.ClearGraph;
  FilterGraph.Active := False;
  Filter.BaseFilter.Moniker := GetVideoDeviceByIndex(Index);

  PinList := TPinList.Create(Filter.Filter);
  i := 0;
  while i < PinList.Count do
    if PinList.PinInfo[i].dir = PINDIR_OUTPUT then begin
      VideoMediaTypes.Assign(PinList.Items[i]);
      PinList.Delete(i);
    end else inc(i);

  PinList.Free;

  FilterGraph.Active := True;
  Open;
end;

procedure TSRVMediaPlayer.SetAudioInDevice(Index : Integer);
var
     PinList: TPinList;
     i: integer;
     ABool: LongBool;
begin
  FilterGraph.Active := False;

  AudioInFilter.BaseFilter.Moniker := GetAudioInDeviceByIndex(Index);
  AudioInFilter.FilterGraph := FilterGraph;
  FilterGraph.Active := True;

  IndexAudioInFormat := -1;
  PinList := TPinList.Create(AudioInFilter.Filter);
  i := 0;
  while i < PinList.Count do
    if PinList.PinInfo[i].dir = PINDIR_OUTPUT then begin
      AudioMediaInTypes.Assign(PinList.Items[i]);
      PinList.Delete(i);
    end else inc(i);

  IndexInputLines := -1;
  for i := 0 to PinList.Count - 1 do begin
    InputLines.Add(PinList.PinInfo[i].achName);
    with (PinList.Items[i] as IAMAudioInputMixer) do get_Enable(ABool);
      if ABool then IndexInputLines := i;
  end;

  If PinList.Count > 0 then
    with (PinList.Items[0] as IAMAudioInputMixer) do
      put_Enable(true);

  PinList.Free;
  FilterGraph.Active := False;

  Open;
end;

procedure TSRVMediaPlayer.SetAudioOutDevice(Index : Integer);
var
  PinList: TPinList;
  i: integer;
begin
  FilterGraph.Active := False;

  AudioOutFilter.BaseFilter.Moniker := GetAudioOutDeviceByIndex(Index);
  AudioOutFilter.FilterGraph := FilterGraph;
  FilterGraph.Active := True;

  PinList := TPinList.Create(AudioOutFilter.Filter);
  i := 0;
  while i < PinList.Count do
    if PinList.PinInfo[i].dir = PINDIR_OUTPUT then begin
      AudioMediaOutTypes.Assign(PinList.Items[i]);
      PinList.Delete(i);
    end else inc(i);

{  IndexOutputLines := -1;
  for i := 0 to PinList.Count - 1 do begin
    OutputLines.Add(PinList.PinInfo[i].achName);
    with (PinList.Items[i] as IAMAudioInputMixer) do get_Enable(ABool);
      if ABool then IndexOutputLines := i;
  end;

{  If PinList.Count > 0 then
    with (PinList.Items[0] as IAMAudioInputMixer) do
      put_Enable(true);}

  PinList.Free;
  FilterGraph.Active := False;

//  FilterGraph.Volume := 100;
//  FilterGraph.Balance := 100;
  Open;
end;

function TSRVMediaPlayer.GetEnumVideoFormat(Items: TStrings) : Boolean;
var
     i: integer;
begin
  Result := True;

  Items.Clear;
  for i := 0 to VideoMediaTypes.Count - 1 do
    Items.Add(VideoMediaTypes.MediaDescription[i]);
end;

function TSRVMediaPlayer.GetEnumAudioInFormat(Items: TStrings) : Boolean;
var
     i: integer;
begin
  Result := True;

  Items.Clear;
  for i := 0 to AudioMediaInTypes.Count - 1 do
    Items.Add(AudioMediaInTypes.MediaDescription[i]);
end;

function TSRVMediaPlayer.GetEnumAudioOutFormat(Items: TStrings) : Boolean;
var
     i: integer;
begin
  Result := True;

  Items.Clear;
  for i := 0 to AudioMediaOutTypes.Count - 1 do
    Items.Add(AudioMediaOutTypes.MediaDescription[i]);
end;

function TSRVMediaPlayer.GetEnumAudioInput(Items: TStrings) : Integer;
begin
  Result := IndexInputLines;
  Items.Assign(InputLines);
end;

procedure TSRVMediaPlayer.SetAudioInFormat(Index : Integer);
begin
  Close;
  IndexAudioInFormat := Index;
  Open;
end;

procedure TSRVMediaPlayer.SetAudioInLines(Index : Integer);
begin
  Close;
  IndexInputLines := Index;
  Open;
end;

//Set the TVfrequency of the TVTuner
function TSRVMediaPlayer.SetFreq(Value: LongWord): HRESULT;
var
   PropertySet : IKSPropertySet;
   fFreq : TKSPropertyTunerFrequencyS;
   fModeCaps: TKSPropertyTunerModeCapsS;
   hr : HRESULT;
   dwSupported, cbBytes: DWord;
begin
  Result:= E_FAIL;
  If not Assigned(MyAMTVTuner) then exit;

  // Query the IKsPropertySet on the Device TV Tuner Filter.
  // pTuner is IBaseFilter Pointer of the TV Tuner Filter.
  try
   hr:= MyAMTVTuner.QueryInterface(IID_IKSPropertySet, PropertySet);
   if Succeeded(hr) then begin

     dwSupported:= 0;
     fModeCaps.Mode:= AMTUNER_MODE_TV; // = AMTUNER_MODE_TV

     // Check if the properties are supported by the tuner minidriver,  used to be: KSPROPERTY_TUNER_MODE_CAPS
     hr:= PropertySet.QuerySupported(PROPSETID_TUNER, AMPROPERTY_PIN_CATEGORY, dwSupported);

     if Succeeded(hr) then begin  //should return S_OK if supported (see: sdk, results for QuerySupported)
       cbBytes:= 0;
       hr := PropertySet.Get(PROPSETID_TUNER,
                             DWORD(KSPROPERTY_TUNER_MODE_CAPS),
                             PKSPROPERTY(@fModeCaps),
                             SizeOf(fModeCaps) - SizeOf(PKSPROPERTY),
                             fModeCaps,
                             SizeOf(fModeCaps),
                             cbBytes);

     end;

     // check if the given frequency is in tuner range
     if Succeeded(hr) and (value >= fModeCaps.MinFrequency) and (value <= fModeCaps.MaxFrequency) then begin
       cbBytes := 0;
       hr := PropertySet.Get(PROPSETID_TUNER,
                             DWORD(KSPROPERTY_TUNER_FREQUENCY),
                             PKSPROPERTY(@fFreq),
                             SizeOf(fFreq) - SizeOf(PKSPROPERTY),
                             fFreq,
                             SizeOf(fFreq),
                             cbBytes);

       if (fModeCaps.Strategy = KS_TUNER_STRATEGY_DRIVER_TUNES) then
         fFreq.TuningFlags:= DWORD(KS_TUNER_TUNING_FINE)
       else
         fFreq.TuningFlags:= DWORD(KS_TUNER_TUNING_EXACT);

       fFreq.Frequency:= Value;
       if Succeeded(hr) then
         hr:= PropertySet.Set_(PROPSETID_TUNER,
                               DWORD(KSPROPERTY_TUNER_FREQUENCY),
                               PKSPROPERTY(@fFreq),
                               SizeOf(fFreq) - SizeOf(PKSPROPERTY),
                               @fFreq,
                               SizeOf(fFreq));

       if Succeeded(hr) then
         result:= S_OK;
     end;
   end;

  finally
    PropertySet:= nil;
  end;
end;

function FindCrossbarPin(
    XBar : IAMCrossbar;              // Pointer to the crossbar.
    PhysicalType : Integer;          // Pin type to match.
    Dir : TPin_Direction;            // Pin direction.
    var Index : Integer) : HRESULT;  // Receives the index of the pin, if found.
var
     bInput : Boolean;
     i, count, cOut, cIn, iRelated, ThisPhysicalType : Integer;
begin
    bInput := Dir = PINDIR_INPUT;

    // Find out how many pins the crossbar has.
    Result := XBar.get_PinCounts(cOut, cIn);
    if Failed(Result) then
      Exit;
    // Enumerate pins and look for a matching pin.
    If bInput then count := cIn
    else count := cOut;

    for i := 0 to count-1 do begin
      iRelated := 0;
      ThisPhysicalType := 0;
      Result := XBar.get_CrossbarPinInfo(bInput, i, iRelated, ThisPhysicalType);
      if Succeeded(Result) and (ThisPhysicalType = Integer(PhysicalType)) then begin
        // Found a match, return the index.
        Index := i;
        Result := S_OK;
        Exit;
      end;
    end;

    // Did not find a matching pin.
    Result := E_FAIL;
end;

function ConnectVideoTunner(XBar : IAMCrossbar; bActivate : Boolean) : HRESULT;
var
     i, j : Integer;
begin
    // Look for the Video Decoder output pin.
    i := 0;
    Result := FindCrossbarPin(XBar, PhysConn_Video_VideoDecoder, PINDIR_OUTPUT, i);
    if Succeeded(Result) then begin
      if bActivate then begin // Activate the Video.
        // Look for the Video Tuner input pin.
        j := 0;
        Result := FindCrossbarPin(XBar, PhysConn_Video_Tuner, PINDIR_INPUT, j);
        if Succeeded(Result) then begin
          Result := XBar.Route(i, j);
          Exit;
        end;
      end else begin // Mute the video
        Result := XBar.Route(i, -1);
        Exit;
      end;
    end;
    Result := E_FAIL;
end;

function ConnectAudioTunner(XBar : IAMCrossbar; bActivate : Boolean) : HRESULT;
var
     i, j : Integer;
begin
    // Look for the Audio Decoder output pin.
    i := 0;
    Result := FindCrossbarPin(XBar, PhysConn_Audio_AudioDecoder, PINDIR_OUTPUT, i);
    if Succeeded(Result) then begin
      if bActivate then begin // Activate the audio.
        // Look for the Audio Tuner input pin.
        j := 0;
        Result := FindCrossbarPin(XBar, PhysConn_Audio_Tuner, PINDIR_INPUT, j);
        if Succeeded(Result) then begin
          Result := XBar.Route(i, j);
          Exit;
        end;
      end else begin // Mute the audio
        Result := XBar.Route(i, -1);
        Exit;
      end;
    end;
    Result := E_FAIL;
end;

function ActivateVideo(Build : ICaptureGraphBuilder2; Src : IBaseFilter;
  bActivate : Boolean) : HRESULT;
var
     XBar1, XBar2 : IAMCrossbar;
     F : IBaseFilter;
begin
   Result := E_FAIL;
   If Build = nil then exit;
   // Search upstream for a crossbar.
   XBar1 := nil;
   Result := Build.FindInterface(@LOOK_UPSTREAM_ONLY, nil, Src, IID_IAMCrossbar, XBar1);
   if Succeeded(Result) then begin
     Result := ConnectVideoTunner(XBar1, bActivate);
     if Failed(Result) then begin
       // Look for another crossbar.
       F := nil;
       Result := XBar1.QueryInterface(IID_IBaseFilter, F);
       if Succeeded(Result) then begin
         // Search upstream for another one.
         XBar2 := nil;
         Result := Build.FindInterface(@LOOK_UPSTREAM_ONLY, nil, F,
             IID_IAMCrossbar, XBar2);
         F._Release;
         F := nil;
         if Succeeded(Result) then begin
           Result := ConnectVideoTunner(XBar2, bActivate);
           XBar2._Release;
           XBar2 := nil;
         end;
       end;
     end;
     XBar1._Release;
     XBar1 := nil;
    end;
end;

function ActivateAudio(Build : ICaptureGraphBuilder2; Src : IBaseFilter;
  bActivate : Boolean) : HRESULT;
var
     XBar1, XBar2 : IAMCrossbar;
     F : IBaseFilter;
begin
  Result := E_FAIL;
  If Build = nil then exit;
  // Search upstream for a crossbar.
   XBar1 := nil;
   Result := Build.FindInterface(@LOOK_UPSTREAM_ONLY, nil, Src, IID_IAMCrossbar, XBar1);
   if Succeeded(Result) then begin
     Result := ConnectAudioTunner(XBar1, bActivate);
     if Failed(Result) then begin
       // Look for another crossbar.
       F := nil;
       Result := XBar1.QueryInterface(IID_IBaseFilter, F);
       if Succeeded(Result) then begin
         // Search upstream for another one.
         XBar2 := nil;
         Result := Build.FindInterface(@LOOK_UPSTREAM_ONLY, nil, F,
             IID_IAMCrossbar, XBar2);
         F._Release;
         F := nil;
         if Succeeded(Result) then begin
           Result := ConnectAudioTunner(XBar2, bActivate);
           XBar2._Release;
           XBar2 := nil;
         end;
       end;
     end;
     XBar1._Release;
     XBar1 := nil;
    end;
end;

function  TSRVMediaPlayer.SetTVVideo(Value : Boolean) : Boolean;
begin
  // Build the analog TV graph (not shown).
  // Activate the video.
  Result := Succeeded(ActivateVideo(FilterGraph.CaptureGraph, Filter.Filter, Value));
end;

function  TSRVMediaPlayer.SetTVAudio(Value : Boolean) : Boolean;
begin
  // Build the analog TV graph (not shown).
  // Activate the audio.
  Result := Succeeded(ActivateAudio(FilterGraph.CaptureGraph, Filter.Filter, Value));
end;

function TSRVMediaPlayer.AvailableTVFormats : TSRVVideoFormats;
var
     VideoFormat : Integer;
begin
  Result := [];
  If not Assigned(MyAMTVTuner) then exit;

  MyAMTVTuner.get_AvailableTVFormats(VideoFormat);
  If (VideoFormat and AnalogVideo_None) <> 0 then
    Result := Result + [vfNone];

  If (VideoFormat and AnalogVideo_NTSC_M) <> 0 then
    Result := Result + [vfNTSC_M];

  If (VideoFormat and AnalogVideo_NTSC_M_J) <> 0 then
    Result := Result + [vfNTSC_M_J];

  If (VideoFormat and AnalogVideo_NTSC_433) <> 0 then
    Result := Result + [vfNTSC_433];

  If (VideoFormat and AnalogVideo_PAL_B) <> 0 then
    Result := Result + [vfPAL_B];

  If (VideoFormat and AnalogVideo_PAL_D) <> 0 then
    Result := Result + [vfPAL_D];

  If (VideoFormat and AnalogVideo_PAL_G) <> 0 then
    Result := Result + [vfPAL_G];

  If (VideoFormat and AnalogVideo_PAL_H) <> 0 then
    Result := Result + [vfPAL_H];

  If (VideoFormat and AnalogVideo_PAL_I) <> 0 then
    Result := Result + [vfPAL_I];

  If (VideoFormat and AnalogVideo_PAL_M) <> 0 then
    Result := Result + [vfPAL_M];

  If (VideoFormat and AnalogVideo_PAL_N) <> 0 then
    Result := Result + [vfPAL_N];

  If (VideoFormat and AnalogVideo_PAL_60) <> 0 then
    Result := Result + [vfPAL_60];

  If (VideoFormat and AnalogVideo_SECAM_B) <> 0 then
    Result := Result + [vfSECAM_B];

  If (VideoFormat and AnalogVideo_SECAM_D) <> 0 then
    Result := Result + [vfSECAM_D];

  If (VideoFormat and AnalogVideo_SECAM_G) <> 0 then
    Result := Result + [vfSECAM_G];

  If (VideoFormat and AnalogVideo_SECAM_H) <> 0 then
    Result := Result + [vfSECAM_H];

  If (VideoFormat and AnalogVideo_SECAM_K) <> 0 then
    Result := Result + [vfSECAM_K];

  If (VideoFormat and AnalogVideo_SECAM_K1) <> 0 then
    Result := Result + [vfSECAM_K1];

  If (VideoFormat and AnalogVideo_SECAM_L) <> 0 then
    Result := Result + [vfSECAM_L];

  If (VideoFormat and AnalogVideo_SECAM_L1) <> 0 then
    Result := Result + [vfSECAM_L1];
end;

function TSRVMediaPlayer.TVFormat : TSRVVideoFormat;
var
     VideoFormat : Integer;
begin
  Result := vfNone;
  If not Assigned(MyAMTVTuner) then exit;

  MyAMTVTuner.get_TVFormat(VideoFormat);
  case VideoFormat of
    AnalogVideo_None: Result := vfNone;
    AnalogVideo_NTSC_M: Result := vfNTSC_M;
    AnalogVideo_NTSC_M_J: Result := vfNTSC_M_J;
    AnalogVideo_NTSC_433: Result := vfNTSC_433;
    AnalogVideo_PAL_B: Result := vfPAL_B;
    AnalogVideo_PAL_D: Result := vfPAL_D;
    AnalogVideo_PAL_G: Result := vfPAL_G;
    AnalogVideo_PAL_H: Result := vfPAL_H;
    AnalogVideo_PAL_I: Result := vfPAL_I;
    AnalogVideo_PAL_M: Result := vfPAL_M;
    AnalogVideo_PAL_N: Result := vfPAL_N;
    AnalogVideo_PAL_60: Result := vfPAL_60;
    AnalogVideo_SECAM_B: Result := vfSECAM_B;
    AnalogVideo_SECAM_D: Result := vfSECAM_D;
    AnalogVideo_SECAM_G: Result := vfSECAM_G;
    AnalogVideo_SECAM_H: Result := vfSECAM_H;
    AnalogVideo_SECAM_K: Result := vfSECAM_K;
    AnalogVideo_SECAM_K1: Result := vfSECAM_K1;
    AnalogVideo_SECAM_L: Result := vfSECAM_L;
    AnalogVideo_SECAM_L1: Result := vfSECAM_L1;
  end;
end;

function TSRVMediaPlayer.TVFormatStr : String;
var
     VideoFormat : Integer;
begin
  Result := 'None';
  If not Assigned(MyAMTVTuner) then exit;

  MyAMTVTuner.get_TVFormat(VideoFormat);
  case VideoFormat of
    AnalogVideo_None: Result := 'None';
    AnalogVideo_NTSC_M: Result := 'NTSC_M';
    AnalogVideo_NTSC_M_J: Result := 'NTSC_M_J';
    AnalogVideo_NTSC_433: Result := 'NTSC_433';
    AnalogVideo_PAL_B: Result := 'PAL_B';
    AnalogVideo_PAL_D: Result := 'PAL_D';
    AnalogVideo_PAL_G: Result := 'PAL_G';
    AnalogVideo_PAL_H: Result := 'PAL_H';
    AnalogVideo_PAL_I: Result := 'PAL_I';
    AnalogVideo_PAL_M: Result := 'PAL_M';
    AnalogVideo_PAL_N: Result := 'PAL_N';
    AnalogVideo_PAL_60: Result := 'PAL_60';
    AnalogVideo_SECAM_B: Result := 'SECAM_B';
    AnalogVideo_SECAM_D: Result := 'SECAM_D';
    AnalogVideo_SECAM_G: Result := 'SECAM_G';
    AnalogVideo_SECAM_H: Result := 'SECAM_H';
    AnalogVideo_SECAM_K: Result := 'SECAM_K';
    AnalogVideo_SECAM_K1: Result := 'SECAM_K1';
    AnalogVideo_SECAM_L: Result := 'SECAM_L';
    AnalogVideo_SECAM_L1: Result := 'SECAM_L1';
  end;
end;


procedure TSRVMediaPlayer.RadioTunerBeginScan;
var
   KsBase : IKSPropertySet;
   hr : HRESULT;
   v : Integer;
begin
  hr:= Filter.Filter.QueryInterface(IID_IKsPropertySet, KsBase);

  If Succeeded(hr) then begin
    v := 1;
    KsBase.Set_(KSPROPSETID_ANALOG_TUNER_API, KSPROPERTY_ID_TUNER_SCAN_MODE, nil,
                0, @v, SizeOf(v));

    KsBase._Release();
    KsBase := nil;
  end;
end;

procedure TSRVMediaPlayer.RadioTunerEndScan;
var
   KsBase : IKSPropertySet;
   hr : HRESULT;
   v : Integer;
begin
  hr := Filter.Filter.QueryInterface(IID_IKsPropertySet, KsBase);

  If Succeeded(hr) then begin
    v := 0;
    KsBase.Set_(KSPROPSETID_ANALOG_TUNER_API, KSPROPERTY_ID_TUNER_SCAN_MODE, nil,
                0, @v, SizeOf(v));

    KsBase._Release();
    KsBase := nil;
  end;
end;

procedure TSRVMediaPlayer.WMPaint(var Message: TWMPaint);
begin
//  PaintTo(Message.DC);

//  if Message.DC = 0 then
    Inherited;
end;

procedure TSRVMediaPlayer.PlayDVD(FileName: WideString; Play : Boolean = True);
var
  Status : TAM_DVD_RenderStatus;
  DvdCmd: IDvdCmd;
  HR: HRESULT;
  DVDControl2: IDVDControl2;
begin
  // Activate the filter Graph
  if not FilterGraph.Active then
  begin
    FilterGraph.Active := true;
    // Render DVD
    HR := FilterGraph.RenderDvd(Status, FileName);
    if HR <> S_OK then
    begin
      case HR of
        HRESULT(E_INVALIDARG): Application.MessageBox('Invalid Argument.','Error', mb_ok);
        HRESULT(S_FALSE)     : begin
                        if Status.hrVPEStatus <> 0 then Application.MessageBox(PChar(GetErrorString(Status.hrVPEStatus)), 'Error', mb_OK);
                        if Status.bDvdVolInvalid   then Application.MessageBox('The specified DVD volume to be played does not exist.', 'Error', mb_OK);
                        if Status.bDvdVolUnknown   then Application.MessageBox('No DVD volume is specified or isn''t found.', 'Error', mb_OK);
                        if Status.bNoLine21In      then Application.MessageBox('The video decoder doesn''t produce line 21 (closed captioning) data.', 'Error', mb_OK);
                        if Status.bNoLine21Out     then Application.MessageBox('The video decoder can''t be shown as closed captioning on video due to a problem with graph building.', 'Error', mb_OK);
                        if status.iNumStreamsFailed > 0 then Application.MessageBox('Can''t render one or more stream.', 'Error', mb_OK);
                      end;
        HRESULT(VFW_E_DVD_DECNOTENOUGH) : Application.MessageBox('There isn''t enough hardware or software decoders to decode all streams.','Error', mb_OK);
        HRESULT(VFW_E_DVD_RENDERFAIL)   : Application.MessageBox('Some basic error occurred in building the graph.'#13'Possibilities include the DVD Navigator filter or the video or audio renderer not instantiating,'#13'a trivial connection or pin enumeration failing, or none of the streams rendering.','Error', mb_OK);
      end;
      FilterGraph.ClearGraph;
      exit;
    end;
    If Play then
      FilterGraph.Play;

    If Succeeded(FilterGraph.QueryInterface(IID_IDVDControl2, DVDControl2)) then
    begin
      DVDControl2.SetOption(DVD_NotifyParentalLevelChange,false); //not notify us when parental level changes
      DVDControl2.SetOption(DVD_HMSF_TimeCodeEvents, true);       // use new HMSF timecode format
    end;
  end
  else
  begin
    FilterGraph.Play;
    FDVDSpeed := 1;
    If Succeeded(FilterGraph.QueryInterface(IID_IDVDControl2, DVDControl2)) then
      DVDControl2.PlayForwards(1.0, DVD_CMD_FLAG_None, DvdCmd);
  end;
  DVDControl2 := nil;
end;

procedure TSRVMediaPlayer.DVDStepForward;
var
  FrameStep: IVideoFrameStep;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IVideoFrameStep, FrameStep);
    FrameStep.Step(1, nil);
    FrameStep := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDRootMenu;
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    DvdControl.ShowMenu(DVD_MENU_Root, 0, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDSavebookmark(FileName: String);
begin
  FilterGraph.DVDSaveBookmark(FileName);
end;

procedure TSRVMediaPlayer.DVDRestorebookmark(FileName: String);
begin
  FilterGraph.DVDRestoreBookmark(FileName);
end;

procedure TSRVMediaPlayer.DVDTitleMenu;
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    DvdControl.ShowMenu(DVD_MENU_Title, DVD_CMD_FLAG_None, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDNextChapter;
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    DvdControl.PlayNextChapter(DVD_CMD_FLAG_None, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDPreviousChapter;
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    DvdControl.PlayPrevChapter(DVD_CMD_FLAG_None, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDSpeed(Speed : Double);
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    FDVDSpeed := Speed;
    If Speed > 0 then
      DvdControl.PlayForwards(Speed, DVD_CMD_FLAG_None, DvdCmd)
    else
      DvdControl.PlayBackwards(-Speed, DVD_CMD_FLAG_None, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDFastForward;
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FDVDSpeed := 8.0;
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    DvdControl.PlayForwards(8.0, DVD_CMD_FLAG_None, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDFastBackward;
var
  DvdCmd: IDvdCmd;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    FDVDSpeed := -8.0;
    DvdControl.PlayBackwards(8.0, DVD_CMD_FLAG_None, DvdCmd);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDMouseMove(X, Y: Integer);
var
  Point : TPoint;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    Point.x := X;
    Point.y := Y;
    DvdControl.SelectAtPosition(Point);
    DvdControl := nil;
  end;
end;

procedure TSRVMediaPlayer.DVDMouseUp(X, Y: Integer);
var
  Point : TPoint;
  DvdControl: IDvdControl2;
begin
  if FilterGraph.Active then
  begin
    FilterGraph.QueryInterface(IDvdControl2, DvdControl);
    Point.x := X;
    Point.y := Y;
    DvdControl.ActivateAtPosition(Point);
    DvdControl := nil;
  end;
end;

function  TSRVMediaPlayer.GetOnDVDAudioStreamChange : TOnDVDAudioStreamChange;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDAudioStreamChange
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDAudioStreamChange(Value : TOnDVDAudioStreamChange);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDAudioStreamChange := Value;
end;

function  TSRVMediaPlayer.GetOnDVDSubpictureStreamChange : TOnDVDSubpictureStreamChange;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDSubpictureStreamChange
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDSubpictureStreamChange(Value : TOnDVDSubpictureStreamChange);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDSubpictureStreamChange := Value;
end;


function  TSRVMediaPlayer.GetOnDVDTitleChange : TOnDVDTitleChange;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDTitleChange
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDTitleChange(Value : TOnDVDTitleChange);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDTitleChange := Value;
end;

function  TSRVMediaPlayer.GetOnDVDChapterStart : TOnDVDChapterStart;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDChapterStart
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDChapterStart(Value : TOnDVDChapterStart);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDChapterStart := Value;
end;

function  TSRVMediaPlayer.GetOnDVDAngleChange : TOnDVDChange;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDAngleChange
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDAngleChange(Value : TOnDVDChange);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDAngleChange := Value;
end;

function  TSRVMediaPlayer.GetOnDVDButtonChange : TOnDVDChange;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDButtonChange
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDButtonChange(Value : TOnDVDChange);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDButtonChange := Value;
end;

function  TSRVMediaPlayer.GetOnDVDPlaybackRateChange : TOnDVDPlaybackRateChange;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDPlaybackRateChange
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDPlaybackRateChange(Value : TOnDVDPlaybackRateChange);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDPlaybackRateChange := Value;
end;

function  TSRVMediaPlayer.GetOnDVDCurrentTime : TOnDVDCurrentTime;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDCurrentTime
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDCurrentTime(Value : TOnDVDCurrentTime);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDCurrentTime := Value;
end;

function  TSRVMediaPlayer.GetOnDVDKaraokeMode : TOnDVDKaraokeMode;
begin
  If Assigned(FilterGraph) then
    Result := FilterGraph.OnDVDKaraokeMode
  else Result := nil;
end;

procedure TSRVMediaPlayer.SetOnDVDKaraokeMode(Value : TOnDVDKaraokeMode);
begin
  If Assigned(FilterGraph) then
    FilterGraph.OnDVDKaraokeMode := Value;
end;

procedure TSRVMediaPlayer.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
Var
     MPos : TPoint;
begin
  if FFullScreen then
    MPos := Point(mouse.CursorPos.X, mouse.CursorPos.Y)
  else
    MPos := Point(X, Y);

  if MyVideoWindow <> nil then begin
    MyVideoWindow.IsCursorHidden(IsHidden);
    if IsHidden then begin
      MyVideoWindow.HideCursor(False);
      MousePos := MPos;
      CursorMov := GetTickCount;
      IsHidden := False;
      if Assigned(FOnCursorVisible) then
        FOnCursorVisible(Self, True);
    end;
  end;

  inherited;
end;

procedure TSRVMediaPlayer.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if FilterGraph.Active and (GraphMode = gmDVD) and
     ((FDisplay = nil) or (FDisplay = Self)) then
    DVDMouseUp(X, Y);

  inherited;
end;

procedure TSRVMediaPlayer.MouseMove(Shift: TShiftState; X, Y: Integer);
  var
    MPos : TPoint;
begin
  if FilterGraph.Active and (GraphMode = gmDVD) and
     ((FDisplay = nil) or (FDisplay = Self)) then
    DVDMouseMove(X, Y);

  if FFullScreen then
    MPos := Point(Mouse.CursorPos.X, Mouse.CursorPos.Y)
  else
    MPos := Point(X, Y);

  if (MousePos.X <> MPos.X) or (MousePos.Y <> MPos.Y) then begin
    MousePos := MPos;
    CursorMov := GetTickCount;
    if MyVideoWindow <> nil then begin
      MyVideoWindow.IsCursorHidden(IsHidden);
      if IsHidden then begin
        MyVideoWindow.HideCursor(False);
        IsHidden := False;
        if Assigned(FOnCursorVisible) then
          FOnCursorVisible(Self, True);
      end;
    end;
  end;

  inherited;
end;

procedure TSRVMediaPlayer.SetZoom(Value : Integer);
var
    Ratio : Real;
    TmpX, TmpY : Real;
    TmpLeft, TmpTop : Real;
begin
  if (Value < 0) or (Value > 99) then begin
    raise Exception.CreateFmt('Value %d out of range. Value must bee between 0 -> 99', [Value]);
    Exit;
  end;

  if (csDesigning in ComponentState) or (MyBasicVideo = nil) then begin
    FZoom := Value;
    Exit;
  end;

  try
    Ratio := DefaultHeight / DefaultWidth;

    TmpX := DefaultWidth - ((Value * Defaultwidth) / 100);
    TmpY := TmpX * Ratio;

    TmpLeft := (DefaultWidth - TmpX) / 2;
    TmpTop := (DefaultHeight - TmpY) / 2;

    MyBasicVideo.put_SourceWidth(Trunc(TmpX));
    MyBasicVideo.put_SourceHeight(Trunc(TmpY));
    MyBasicVideo.put_SourceLeft(Trunc(TmpLeft));
    MyBasicVideo.put_SourceTop(Trunc(TmpTop));
    FZoom := Value;
  finally
  end;
end;

procedure TSRVMediaPlayer.MyIdleHandler(Sender: TObject; var Done: Boolean);
var
    pt : TPoint;
begin
  Done := True;
  if (FIdleCursor = 0) or (csDesigning in ComponentState) then exit;
  if (GetTickCount - CursorMov >= Cardinal(FIdleCursor)) and (MyVideoWindow <> nil) then begin
    MyVideoWindow.IsCursorHidden(IsHidden);
    if not IsHidden then begin
      MyVideoWindow.HideCursor(True);
      IsHidden := True;
      GetCursorPos(pt);
      SetCursorPos(pt.x, pt.y);
      if Assigned(FOnCursorVisible) then
        FOnCursorVisible(Self, False);
    end;
  end;
end;

procedure TSRVMediaPlayer.EnabledAllSoundStream(Volume : Integer; Enabled : Boolean);
var
     EnumFilters: IEnumFilters; {The Variables needed for Filter Operations}
     BaseFilter: IBaseFilter;
     FilterInfo: TFilterInfo;
     Klasse:TGuid;
begin
  If (GraphMode <> gmDVD) then begin
    FilterGraph.FilterGraph.EnumFilters(EnumFilters); {Enumerate Filters}
    While (EnumFilters.Next(1, BaseFilter, nil) = S_OK) do begin
      BaseFilter.QueryFilterInfo(FilterInfo);
      BaseFilter.GetClassID(Klasse);
      If (Klasse.D2 = CLSID_DSoundRender.D2) and
         (Klasse.D3=CLSID_DSoundRender.D3) then begin {cool Sound Renderer Check}
        If Enabled then
          (BaseFilter as IBasicAudio).put_Volume(FVolume)
        else
          (BaseFilter as IBasicAudio).put_Volume(-10000);
      end;
    end;
  end;
end;

procedure TSRVMediaPlayer.EnabledSoundStream(Index, Volume : Integer; Enabled : Boolean);
var
     EnumFilters: IEnumFilters; {The Variables needed for Filter Operations}
     BaseFilter: IBaseFilter;
     FilterInfo: TFilterInfo;
     Klasse:TGuid;
     IndexAudioFilter : Integer;
begin
  IndexAudioFilter := 0;
  FilterGraph.FilterGraph.EnumFilters(EnumFilters); {Enumerate Filters}
  While (EnumFilters.Next(1, BaseFilter, nil) = S_OK) do begin
    BaseFilter.QueryFilterInfo(FilterInfo);
    BaseFilter.GetClassID(Klasse);
    If (Klasse.D2 = CLSID_DSoundRender.D2) and
       (Klasse.D3=CLSID_DSoundRender.D3) then begin {cool Sound Renderer Check}
      If (IndexAudioFilter = Index) then
        If Enabled then
          (BaseFilter as IBasicAudio).put_Volume(FVolume)
        else
          (BaseFilter as IBasicAudio).put_Volume(-10000);
      IndexAudioFilter := IndexAudioFilter + 1;
    end;
  end;
end;

function  TSRVMediaPlayer.GetVolumeSoundStream(Index : Integer) : Integer;
var
     EnumFilters: IEnumFilters; {The Variables needed for Filter Operations}
     BaseFilter: IBaseFilter;
     FilterInfo: TFilterInfo;
     Klasse:TGuid;
     IndexAudioFilter : Integer;
begin
  Result := -10000;
  IndexAudioFilter := 0;
  FilterGraph.FilterGraph.EnumFilters(EnumFilters); {Enumerate Filters}
  While (EnumFilters.Next(1, BaseFilter, nil) = S_OK) do begin
    BaseFilter.QueryFilterInfo(FilterInfo);
    BaseFilter.GetClassID(Klasse);
    If (Klasse.D2 = CLSID_DSoundRender.D2) and
       (Klasse.D3=CLSID_DSoundRender.D3) then begin {cool Sound Renderer Check}
      If (IndexAudioFilter = Index) then
        (BaseFilter as IBasicAudio).get_Volume(Result);
      IndexAudioFilter := IndexAudioFilter + 1;
    end;
  end;
end;

procedure TSRVMediaPlayer.SetSoundStream(Index : Integer; Volume : Integer);
var
     EnumFilters: IEnumFilters; {The Variables needed for Filter Operations}
     BaseFilter: IBaseFilter;
     FilterInfo: TFilterInfo;
     Klasse:TGuid;
     IndexAudioFilter : Integer;
begin
  FilterGraph.FilterGraph.EnumFilters(EnumFilters); {Enumerate Filters}
  IndexAudioFilter := 0;
  While (EnumFilters.Next(1, BaseFilter, nil) = S_OK) do begin
    BaseFilter.QueryFilterInfo(FilterInfo);
    BaseFilter.GetClassID(Klasse);
    If (Klasse.D2 = CLSID_DSoundRender.D2) and
       (Klasse.D3=CLSID_DSoundRender.D3) then begin {cool Sound Renderer Check}
      If (IndexAudioFilter = Index) then
        (BaseFilter as IBasicAudio).put_Volume(FVolume)
      else
        (BaseFilter as IBasicAudio).put_Volume(-10000);
      IndexAudioFilter := IndexAudioFilter + 1;
    end;
  end;
end;

function  TSRVMediaPlayer.GetSoundStreamsCount : Integer;
var
     EnumFilters: IEnumFilters; {The Variables needed for Filter Operations}
     BaseFilter: IBaseFilter;
     FilterInfo: TFilterInfo;
     Klasse:TGuid;
     DVDInfo2: IDVDInfo2;
     CountAudioStream, CurrentAudioStream : Cardinal;
begin
  Result:=0; {plain }
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetCurrentAudio(CountAudioStream, CurrentAudioStream)) then
      Result := CountAudioStream;

    DVDInfo2 := nil;
  end else if Assigned(FilterGraph) and Assigned(FilterGraph.FilterGraph) then begin
    FilterGraph.FilterGraph.EnumFilters(EnumFilters); {Enumerate Filters}
    While (EnumFilters.Next(1, BaseFilter, nil) = S_OK) do begin
      BaseFilter.QueryFilterInfo(FilterInfo);
      BaseFilter.GetClassID(Klasse);
      If (Klasse.D2=CLSID_DSoundRender.D2) and (Klasse.D3=CLSID_DSoundRender.D3) then begin {cool Sound Renderer Check}
        Result := Result + 1; {counting}
      end;
    end;
  end;
end;

function  TSRVMediaPlayer.GetIndexSoundChannal : Integer;
var
     DVDInfo2: IDVDInfo2;
     CountAudioStream, CurrentAudioStream : Cardinal;
begin
  Result := FIndexSoundChannal;
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetCurrentAudio(CountAudioStream, CurrentAudioStream)) then
      Result := Max(Min(CurrentAudioStream, CountAudioStream-1), 0);

    FIndexSoundChannal := Result;
    DVDInfo2 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetIndexSoundChannal(Value : Integer);
var
     DvdControl2: IDvdControl2;
     DvdCmd: IDvdCmd;
begin
  FIndexSoundChannal := Max(Min(Value, SoundChannalsCount), -1);

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IDvdControl2, DvdControl2)) then begin
      DvdControl2.SelectAudioStream(Value, DVD_CMD_FLAG_Flush {DVD_CMD_FLAG_None}, DvdCmd);

      DvdCmd := nil;
      DVDControl2 := nil;
  end else begin
    If Value = -1 then AllSound := True
    else SetSoundStream(Value, FVolume);
  end;
end;

function TSRVMediaPlayer.GetSoundChannalName(iSoundChannal : Integer;
  var ChannalName : string) : Boolean;
var
     DVDInfo2: IDVDInfo2;
     Language : LCID;
     szString: array [0..MAX_PATH] of Char;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetAudioLanguage(iSoundChannal, Language)) then begin
      if GetLocaleInfo(Language, LOCALE_SENGLANGUAGE, szString, MAX_PATH-1) <= 0 then
        szString[0] := #0;
      ChannalName := szString;
      Result := True;
    end;
    DVDInfo2 := nil;
  end else begin
    ChannalName := 'Track' + IntToStr(iSoundChannal+1);
  end;
end;

procedure TSRVMediaPlayer.SetAllSound(Value : Boolean);
begin
  EnabledAllSoundStream(FVolume, Value);
  FAllSound := Value;
  If Value then
    FIndexSoundChannal := -1;
end;

function TSRVMediaPlayer.GetOutputPin(const BF: IBaseFilter): IPin;
var
     EP: IEnumPins;
     PD: TPin_Direction;
begin
  if BF.EnumPins(EP) = S_OK then begin
    while (EP.Next(1, Result, nil) = S_OK) do begin
      if (Result.QueryDirection(PD) = S_OK) and (PD = PINDIR_OUTPUT) then begin
        EP := nil;
        Exit;
      end;
      Result := nil;
    end;
    EP := nil;
  end;
end;

function TSRVMediaPlayer.FindAudioSwith(const FG: IFilterGraph): IBaseFilter;
// Sucht einen Filter, der das Umschalten von Audiospuren erlaubt
var
     EF: IEnumFilters;
     Pin: IPin;
     SS: IAMStreamSelect;
     SCount: Cardinal;
     MT: TAM_Media_Type;
begin
  if FG.EnumFilters(EF) = S_OK then begin
    while (EF.Next(1, Result, nil) = S_OK) do begin
      if Result.QueryInterface(IID_IAMStreamSelect, SS) = S_OK then begin
        if (SS.Count(SCount) = S_OK) and (SCount > 1) then begin
          Pin := GetOutputPin(Result);
          if (Pin <> nil) and (Pin.ConnectionMediaType(MT) = S_OK) then begin
            if IsEqualGUID(MT.majortype, MEDIATYPE_AnalogAudio) or
               IsEqualGUID(MT.majortype, MEDIATYPE_Audio) then begin
              SS := nil;
              EF := nil;
              Pin := nil;
              Exit;
            end;
          end;
          Pin := nil;
        end;
        SS := nil;
      end;
      Result := nil;
    end;
    EF := nil;
  end else
    Result := nil;
end;

procedure TSRVMediaPlayer.SetAudioStream(const Index: Integer);
// Schaltet auf den angegebenen Kanal um
var
     FG: IFilterGraph;
     BF: IBaseFilter;
     SS: IAMStreamSelect;
begin
  if FilterGraph.QueryInterface(IID_IFilterGraph, FG) = S_OK then begin
    BF := FindAudioSwith(FG);
    if BF <> nil then begin
      if (BF.QueryInterface(IID_IAMStreamSelect, SS) = S_OK) and
         (SS.Enable(Index, AMSTREAMSELECTENABLE_ENABLE) <> S_OK) then
        SS := nil;
      BF := nil;
    end;
    FG := nil;
  end;
end;

Procedure Set_TVTuner(CapEnum           : TSysDevEnum;
                      TvGraph           : TFilterGraph;
//                      TvWindow          : TVideoWindow;
                      var TVTuner       : IAMTVTuner;
                      var TVVolume      : IbasicAudio;
                      var DeviceReady   : Boolean);
Const
  DefaultCountry = 62;
  DefaultChanel = 309;
var
  TunerF,CaptureF : IbaseFilter;
  VideoDecoder     : IAMAnalogVideoDecoder;
  TunerFInfo,
  CaptureFInfo      : WideString;
  hr                     : cardinal;
  outpin               : Ipin;

begin
  TvGraph.Active := True;
//Enumerated TVTuner
  capEnum.SelectGUIDCategory(AM_KSCATEGORY_TVTUNER);
  TunerF := CapEnum.GetBaseFilter(0);
  try
    TunerFInfo :=  WideString(CapEnum.Filters[0].FriendlyName);
  except
    DeviceReady := false;
    TvGraph.Active := false;
    Exit;
  end;

//Enumerated TVDevice
   CapEnum.SelectGUIDCategory(AM_KSCATEGORY_CAPTURE);
   CaptureF := CapEnum.GetBaseFilter(3);
   CaptureFInfo := WideString(CapEnum.Filters[3].FriendlyName);

//Set TV System
   CaptureF.QueryInterface(IID_IAMAnalogVideoDecoder, VideoDecoder);
   VideoDecoder.put_TVFormat(AnalogVideo_PAL_B);

//Create TvTuner
   TunerF.QueryInterface(IID_IAMTVTuner, TVTuner);

//Set TV Tuner
   TvTuner.put_InputType(0,TunerInputCable);
   TvTuner.put_TuningSpace(0);

   TvTuner.put_CountryCode(DefaultCountry);
   TvTuner.put_Channel(DefaultChanel,0,0);

   TvGraph.FilterGraph.AddFilter(CaptureF, PwideChar(CaptureFInfo));

//Render Audio Output
   hr := CaptureF.FindPin('3', OutPin);
   if hr = 0 then
     TvGraph.FilterGraph.Render(OutPin);
// now render streams
   with TvGraph.CaptureGraph do
      RenderStream(@PIN_CATEGORY_PREVIEW, nil, CaptureF as IBaseFilter,
//                   nil, TvWindow as IBaseFilter);
                   nil, nil);

   TVGraph.QueryInterface(IBasicAudio,TVVolume);
   DeviceReady := True;
End;

// ----------------- Language metods
function TSRVMediaPlayer.FindFilter(Cid: TGUID): IBaseFilter;
var
     EnumF: IEnumFilters;
     Fetched: ULong;
     Fi: IBaseFilter;
     g: TGuid;
     FFi: TFilterInfo;
begin
  Result:= nil;
  if FilterGraph.FilterGraph = nil then
    Exit;

  FilterGraph.FilterGraph.EnumFilters(EnumF);
  while EnumF.Next(1, Fi, @Fetched)=S_Ok do begin
    Fi.GetClassID(G);
    Fi.QueryFilterInfo(FFi);
    if IsEqualGUID(g, cid) then
      Result:= Fi
    else
      Fi:= nil;
  end;
  EnumF:= nil;
end;

function TSRVMediaPlayer.GetLanguagesCount: Integer;
var
     DVDInfo2: IDVDInfo2;
     CountLangs : Cardinal;
     Langs: array [0..MAX_PATH] of LCID;
     pLangs : LCID;
begin
  Result := 0;
  if FilterGraph.FilterGraph = nil then
    Exit;

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    pLangs := LCID(@Langs);
    If Succeeded(DVDInfo2.GetMenuLanguages(pLangs, MAX_PATH, CountLangs)) then
      Result := CountLangs;
    DVDInfo2 := nil;
  end;
end;

function TSRVMediaPlayer.GetIndexLanguage: Integer;
var
     DVDInfo2: IDVDInfo2;
     SelLangu: Cardinal;
begin
  Result := -1;
  if FilterGraph.FilterGraph = nil then
    Exit;

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetDefaultMenuLanguage(SelLangu)) then
      Result := SelLangu;
    DVDInfo2 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetIndexLanguage(Value: Integer);
var
     DvdControl2: IDvdControl2;
     DvdCmd: IDvdCmd;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IDvdControl2, DvdControl2)) then begin
      DvdControl2.SelectDefaultMenuLanguage(Value);

      DvdCmd := nil;
      DVDControl2 := nil;
  end;
end;

function TSRVMediaPlayer.GetLanguageName(iLanguage : Integer;
  var LanguageName : string) : Boolean;
var
     DVDInfo2: IDVDInfo2;
     Langs: array [0..MAX_PATH] of LCID;
     szString: array [0..MAX_PATH] of Char;
     CountLangs : Cardinal;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetMenuLanguages(Langs[0], MAX_PATH, CountLangs)) then begin
      if GetLocaleInfo(Langs[iLanguage], LOCALE_SENGLANGUAGE, szString, MAX_PATH-1) <= 0 then
        szString[0] := #0;
      LanguageName := szString;
      Result := True;
    end;
    DVDInfo2 := nil;
  end;
end;

// ----------------- DVD metods
function  TSRVMediaPlayer.GetDVDVolumeCount : Integer;
var
     pNumOfVol, pThisVolNum, pNumOfTitles : ULONG;
     pSide: TDVD_Disc_Side;
     DVDInfo2: IDVDInfo2;
begin
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    DVDInfo2.GetDVDVolumeInfo(pNumOfVol, pThisVolNum, pSide, pNumOfTitles);

    DVDInfo2 := nil;
    Result := pNumOfVol;
  end else
    Result := 0;
end;

function  TSRVMediaPlayer.GetDVDCurrentVolume : Integer;
var
     pNumOfVol, pThisVolNum, pNumOfTitles : ULONG;
     pSide: TDVD_Disc_Side;
     DVDInfo2: IDVDInfo2;
begin
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    DVDInfo2.GetDVDVolumeInfo(pNumOfVol, pThisVolNum, pSide, pNumOfTitles);

    DVDInfo2 := nil;
    Result := pThisVolNum;
  end else
    Result := 0;
end;

function  TSRVMediaPlayer.GetDVDSide : TDVD_Disc_Side;
var
     pNumOfVol, pThisVolNum, pNumOfTitles : ULONG;
     DVDInfo2: IDVDInfo2;
begin
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    DVDInfo2.GetDVDVolumeInfo(pNumOfVol, pThisVolNum, Result, pNumOfTitles);

    DVDInfo2 := nil;
  end else
    Result := DVD_SIDE_INVALID_0;
end;

function  TSRVMediaPlayer.GetDVDTitleCount : Integer;
var
     pNumOfVol, pThisVolNum, pNumOfTitles : ULONG;
     pSide: TDVD_Disc_Side;
     DVDInfo2: IDVDInfo2;
begin
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    DVDInfo2.GetDVDVolumeInfo(pNumOfVol, pThisVolNum, pSide, pNumOfTitles);

    DVDInfo2 := nil;
    Result := pNumOfTitles;
  end else
    Result := 0;
end;

function  TSRVMediaPlayer.GetDVDNumberOfChapters(TitleIndex : Integer) : Integer;
var
     DVDInfo2: IDVDInfo2;
     pulNumOfChapters : ULONG;
begin
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    DVDInfo2.GetNumberOfChapters(TitleIndex, pulNumOfChapters);

    DVDInfo2 := nil;
    Result := pulNumOfChapters;
  end else
    Result := 0;
end;

function  TSRVMediaPlayer.DVDPlayTitle(TitleIndex : Integer) : Boolean;
var
     DVDControl2 : IDVDControl2;
     DvdCmd: IDvdCmd;
begin
  Result := False;
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDControl2, DVDControl2)) then begin

    Result := Succeeded(DVDControl2.Playtitle(TitleIndex, DVD_CMD_FLAG_None, DvdCmd));
    DVDControl2 := nil;
    DvdCmd := nil;
  end;
end;

function  TSRVMediaPlayer.DVDPlayChapterInTitle(TitleIndex, ChapterIndex : Integer) : Boolean;
var
     DVDControl2 : IDVDControl2;
     DvdCmd: IDvdCmd;
begin
  Result := False;
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDControl2, DVDControl2)) then begin

    Result := Succeeded(DvdControl2.PlayChapterInTitle(TitleIndex, ChapterIndex, DVD_CMD_FLAG_None, DvdCmd));
    DVDControl2 := nil;
    DvdCmd := nil;
  end;
end;

// ----------------- Subtitle metods
function TSRVMediaPlayer.IsSubtitlesUsed: Boolean;
var
     BF: IBaseFilter;
     DvdControl2 : IDvdControl2;
begin
  BF := FindFilter(CLSID_DVobSubDecoder);
  Result:= BF <> nil;
  if (not Result) and (GraphMode = gmDVD) then
    Result := Succeeded(FilterGraph.QueryInterface(IDvdControl2, DvdControl2));
  DvdControl2 := nil;
  BF := nil;
end;

function  TSRVMediaPlayer.GetShowSubtitles : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     b : Bool;
     DVDInfo2: IDVDInfo2;
     pulStreamsAvailable, pulCurrentStream: ULONG;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then begin
    If (GraphMode = gmDVD) and
       Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
      If Succeeded(DVDInfo2.GetCurrentSubpicture(pulStreamsAvailable, pulCurrentStream, b)) then
        Result := not b;
      DVDInfo2 := nil;
    end;
    Exit;
  end;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_HideSubtitles(b);
  Result := not b;

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.SetShowSubtitles(Value : Boolean);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     DvdControl2: IDvdControl2;
     DvdCmd: IDvdCmd;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then begin
    If (GraphMode = gmDVD) and
       Succeeded(FilterGraph.QueryInterface(IDvdControl2, DvdControl2)) then begin
        DvdControl2.SetSubpictureState(Value, DVD_CMD_FLAG_Flush {DVD_CMD_FLAG_None}, DvdCmd);
        DvdCmd := nil;
        DVDControl2 := nil;
    end;
    Exit;
  end;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.put_HideSubtitles(not Value);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetSubtitleLanguagesCount : Integer;
var
     DVDInfo2: IDVDInfo2;
     CountStream, CurrentStream : Cardinal;
     pbIsDisabled: BOOL;
begin
  Result:=0;
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetCurrentSubpicture(CountStream, CurrentStream, pbIsDisabled)) then
      Result := CountStream;
    DVDInfo2 := nil;
  end;
end;

function  TSRVMediaPlayer.GetIndexSubtitleLanguage : Integer;
var
     DVDInfo2: IDVDInfo2;
     CountStream, CurrentStream : Cardinal;
     pbIsDisabled: BOOL;
begin
  Result := FIndexSubtitleLanguage;
  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetCurrentSubpicture(CountStream, CurrentStream, pbIsDisabled)) then
      Result := CurrentStream;
    FIndexSubtitleLanguage := Result;
    DVDInfo2 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetIndexSubtitleLanguage(Value : Integer);
var
     DvdControl2: IDvdControl2;
     DvdCmd: IDvdCmd;
begin
  FIndexSubtitleLanguage := Max(Min(Value, SubtitleLanguagesCount), -1);

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IDvdControl2, DvdControl2)) then begin
      DvdControl2.SelectSubpictureStream(Value, DVD_CMD_FLAG_Flush {DVD_CMD_FLAG_None}, DvdCmd);

      DvdCmd := nil;
      DVDControl2 := nil;
  end;
end;

function TSRVMediaPlayer.GetSubtitleLanguageName(iSubtitleLanguage : Integer;
  var LanguageName : PWideChar) : Boolean;
var
     DVDInfo2: IDVDInfo2;
     Language : LCID;
     szString: array [0..MAX_PATH] of Char;
     szWString: array[0..MAX_PATH] of WideChar;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;

  If (GraphMode = gmDVD) and
     Succeeded(FilterGraph.QueryInterface(IID_IDVDInfo2, DVDInfo2)) then begin
    If Succeeded(DVDInfo2.GetSubpictureLanguage(iSubtitleLanguage, Language)) then begin
      if GetLocaleInfo(Language, LOCALE_SENGLANGUAGE, szString, MAX_PATH-1) <= 0 then
        szString[0] := #0;
      StringToWideChar(szString, szWString, MAX_PATH);
      LanguageName := szWString;
      Result := True;
    end;
    DVDInfo2 := nil;
  end;
end;

function  TSRVMediaPlayer.GetFileNameSubtitles : String;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     filename : array[0..10000] of WideChar;
begin
  Result := '';
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IID_IDirectVobSub, Setup);
  ZeroMemory(@filename, SizeOf(filename));
  Setup.get_FileName(filename);
  Result := filename;

  Setup:= nil;
  DVobSub:= nil;
end;

procedure TSRVMediaPlayer.SetFileNameSubtitles(Value : String);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.put_FileName(StringToOleStr(Value));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetSubtitleTextSettings(var lf:LOGFONT; var color:COLORREF; var fShadow, fOutline, fAdvancedRenderer : Bool) : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then
    Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.get_TextSettings(lf, sizeof(lf), color, fShadow, fOutline, fAdvancedRenderer));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.SetSubtitleTextSettings(lf : LOGFONT; color : COLORREF; fShadow, fOutline, fAdvancedRenderer : Bool) : Boolean;
var
     DVobSub: IBasefilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then
    Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.put_TextSettings(lf, sizeof(lf), color,fShadow,fOutline,fAdvancedRenderer));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetFlipTextSubtitles : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     fPicture, fSubtitles : Bool;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_Flip(fPicture, fSubtitles);
  Result := fSubtitles;

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.SetFlipTextSubtitles(Value : Boolean);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     fPicture, fSubtitles : Bool;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_Flip(fPicture, fSubtitles);
  Setup.put_Flip(fPicture, Value);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetFlipPictureSubtitles : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     fPicture, fSubtitles : Bool;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_Flip(fPicture, fSubtitles);
  Result := fPicture;

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.SetFlipPictureSubtitles(Value : Boolean);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     fPicture, fSubtitles : Bool;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_Flip(fPicture, fSubtitles);
  Setup.put_Flip(Value, fSubtitles);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetRectSubtitles : TNormalizedRect;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  ZeroMemory(@Result, SizeOf(Result));
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_ZoomRect(Result);

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.SetRectSubtitles(Value : TNormalizedRect);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.put_ZoomRect(@Value);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.HasConfigDialogSubtitles(iSelected : Integer) : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.HasConfigDialog(iSelected));

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.ShowConfigDialogSubtitles(iSelected : Integer; hWndParent : THandle);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.ShowConfigDialog(iSelected, hWndParent);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetColorFormatSubtitles : Integer;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  ZeroMemory(@Result, SizeOf(Result));
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_ColorFormat(Result);

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.SetColorFormatSubtitles(Value : Integer);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.put_ColorFormat(Value);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetSubtitleTiming(var delay, speedmul, speeddiv : Integer) : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.get_SubtitleTiming(delay, speedmul, speeddiv));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.SetSubtitleTiming(delay, speedmul, speeddiv : Integer) : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.put_SubtitleTiming(delay, speedmul, speeddiv));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.UpdateRegistrySubtitle : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.UpdateRegistry);

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetSubtitlePlacement(var fOverridePlacement : Bool; var xperc, yperc : Integer) : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.get_Placement(fOverridePlacement, xperc, yperc));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.SetSubtitlePlacement(fOverridePlacement : Bool; xperc, yperc : Integer) : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Result := Succeeded(Setup.put_Placement(fOverridePlacement, xperc, yperc));

  DVobSub:= nil;
  Setup:= nil;
end;

function  TSRVMediaPlayer.GetOSDSubtitle : Boolean;
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
     b : Bool;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.get_OSD(b);
  Result := b;

  DVobSub:= nil;
  Setup:= nil;
end;

procedure TSRVMediaPlayer.SetOSDSubtitle(Value : Boolean);
var
     DVobSub: IbaseFilter;
     Setup: IDirectVobSub;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;
  DVobSub := FindFilter(CLSID_DVobSubDecoder);
  if DVobSub = nil then Exit;

  DVobSub.QueryInterface(IDirectVobSub, Setup);
  Setup.put_OSD(Value);

  DVobSub:= nil;
  Setup:= nil;
end;

function TSRVMediaPlayer.ShowFilterPropertyPage(parent: THandle; Filter: IBaseFilter;
  PropertyPage: TPropertyPage = ppDefault): HRESULT;
var
     SpecifyPropertyPages : ISpecifyPropertyPages;
     CaptureDialog : IAMVfwCaptureDialogs;
     CompressDialog: IAMVfwCompressDialogs;
     CAGUID  :TCAGUID;
     FilterInfo: TFilterInfo;
     Code: Integer;
begin
  result := S_FALSE;
  code := 0;
  if Filter = nil then exit;

  ZeroMemory(@FilterInfo, SizeOf(TFilterInfo));

  case PropertyPage of
    ppVFWCapDisplay: code := VfwCaptureDialog_Display;
    ppVFWCapFormat : code := VfwCaptureDialog_Format;
    ppVFWCapSource : code := VfwCaptureDialog_Source;
    ppVFWCompConfig: code := VfwCompressDialog_Config;
    ppVFWCompAbout : code := VfwCompressDialog_About;
  end;

  case PropertyPage of
    ppDefault:
      begin
        result := Filter.QueryInterface(IID_ISpecifyPropertyPages, SpecifyPropertyPages);
        if result <> S_OK then exit;
        result := SpecifyPropertyPages.GetPages(CAGUID);
        if result <> S_OK then exit;
        result := Filter.QueryFilterInfo(FilterInfo);
        if result = S_OK then begin
          result := OleCreatePropertyFrame(parent, 0, 0, FilterInfo.achName, 1, @Filter, CAGUID.cElems, CAGUID.pElems, 0, 0, nil );
          FilterInfo.pGraph := nil;
        end;
        if Assigned(CAGUID.pElems) then CoTaskMemFree(CAGUID.pElems);
        SpecifyPropertyPages := nil;
      end;
    ppVFWCapDisplay..ppVFWCapSource:
      begin
        result := Filter.QueryInterface(IID_IAMVfwCaptureDialogs,CaptureDialog);
        if (result <> S_OK) then exit;
        result := CaptureDialog.HasDialog(code);
        if result <> S_OK then exit;
        result := CaptureDialog.ShowDialog(code,parent);
        CaptureDialog := nil;
      end;
    ppVFWCompConfig..ppVFWCompAbout:
      begin
        result := Filter.QueryInterface(IID_IAMVfwCompressDialogs, CompressDialog);
        if (result <> S_OK) then exit;
        case PropertyPage of
          ppVFWCompConfig: result := CompressDialog.ShowDialog(VfwCompressDialog_QueryConfig, 0);
          ppVFWCompAbout : result := CompressDialog.ShowDialog(VfwCompressDialog_QueryAbout, 0);
        end;
        if result = S_OK then result := CompressDialog.ShowDialog(code,parent);
        CompressDialog := nil;
      end;
  end;
end;

procedure TSRVMediaPlayer.ShowDeviceDialog(PropertyPage: TPropertyPage = ppDefault);
begin
//  ShowFilterPropertyPage(Handle, Filter.BaseFilter.CreateFilter, PropertyPage);
  ShowFilterPropertyPage(Handle, Filter.Filter, PropertyPage);
end;

// ----------------- Close Caption Text (CC)
function TSRVMediaPlayer.IsCCTextUsed: Boolean;
var
     Line21 : IAMLine21Decoder;
begin
  Result := Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21));
  Line21 := nil;
end;

function  TSRVMediaPlayer.GetShowCCText : Boolean;
var
     Line21 : IAMLine21Decoder;
     State : TAM_Line21_CCState;
begin
  Result := False;
  if FilterGraph.FilterGraph = nil then
    Exit;

  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    Line21.GetServiceState(State);
    Result := State = AM_L21_CCSTATE_On;
    Line21 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetShowCCText(Value : Boolean);
var
     Line21 : IAMLine21Decoder;
     State : TAM_Line21_CCState;
begin
  if FilterGraph.FilterGraph = nil then
    Exit;

  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    If Value then State := AM_L21_CCSTATE_On
    else State := AM_L21_CCSTATE_Off;
    Line21.SetServiceState(State);
    Line21 := nil;
  end;
end;

function TSRVMediaPlayer.GetBackgroundColorCCText: TColor;
var
     Line21 : IAMLine21Decoder;
     pdwPhysColor: DWORD;
begin
  Result := clNone;
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    If Succeeded(Line21.GetBackgroundColor(pdwPhysColor)) then
      Result := pdwPhysColor;
    Line21 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetBackgroundColorCCText(Value: TColor);
var
     Line21 : IAMLine21Decoder;
begin
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    Line21.SetBackgroundColor(Value);
    Line21 := nil;
  end;
end;

function TSRVMediaPlayer.GetBackgroundModeCCText: TBGModeSubtitle;
var
     Line21 : IAMLine21Decoder;
     lpMode: TAM_Line21_DrawBGMode;
begin
  Result := bgmsOpaque;
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    If Succeeded(Line21.GetDrawBackgroundMode(lpMode)) then
      If lpMode = AM_L21_DRAWBGMODE_Transparent then
        Result := bgmsTransparent;
    Line21 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetBackgroundModeCCText(Value: TBGModeSubtitle);
var
     Line21 : IAMLine21Decoder;
     lpMode: TAM_Line21_DrawBGMode;
begin
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    If Value = bgmsOpaque then lpMode := AM_L21_DRAWBGMODE_Opaque
    else lpMode := AM_L21_DRAWBGMODE_Transparent;
    Line21.SetDrawBackgroundMode(lpMode);
    Line21 := nil;
  end;
end;

function  TSRVMediaPlayer.GetServiceCCText: TCCTextService;
var
     Line21 : IAMLine21Decoder;
     lpService: TAM_Line21_CCService;
begin
  Result := stsNone;
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    If Succeeded(Line21.GetCurrentService(lpService)) then
      case lpService of
        AM_L21_CCSERVICE_None: Result := stsNone;
        AM_L21_CCSERVICE_Caption1: Result := stsCaption1;
        AM_L21_CCSERVICE_Caption2: Result := stsCaption2;
        AM_L21_CCSERVICE_Text1: Result := stsText1;
        AM_L21_CCSERVICE_Text2: Result := stsText2;
        AM_L21_CCSERVICE_XDS: Result := stsXDS;
        AM_L21_CCSERVICE_INVALID_6: Result := stsInvalid;
        AM_L21_CCSERVICE_INVALID_7: Result := stsInvalid;
        AM_L21_CCSERVICE_INVALID_8: Result := stsInvalid;
        AM_L21_CCSERVICE_INVALID_9: Result := stsInvalid;
        AM_L21_CCSERVICE_DefChannel: Result := stsDefChannel;
      else
        Result := stsInvalid;
      end;
    Line21 := nil;
  end;
end;

procedure TSRVMediaPlayer.SetServiceCCText(Value: TCCTextService);
var
     Line21 : IAMLine21Decoder;
     lpService: TAM_Line21_CCService;
begin
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    case Value of
      stsCaption1: lpService := AM_L21_CCSERVICE_Caption1;
      stsCaption2: lpService := AM_L21_CCSERVICE_Caption2;
      stsText1: lpService := AM_L21_CCSERVICE_Text1;
      stsText2: lpService := AM_L21_CCSERVICE_Text2;
      stsXDS: lpService := AM_L21_CCSERVICE_XDS;
      stsInvalid: lpService := AM_L21_CCSERVICE_Invalid;
      stsDefChannel: lpService := AM_L21_CCSERVICE_DefChannel;
    else
      lpService := AM_L21_CCSERVICE_None;
    end;
    Line21.SetCurrentService(lpService);
    Line21 := nil;
  end;
end;

function TSRVMediaPlayer.GetOutputFormatCCText(var lpbmih: TBitmapInfoHeader) : Boolean;
var
     Line21 : IAMLine21Decoder;
begin
  Result := False;
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    Result := Succeeded(Line21.GetOutputFormat(lpbmih));
    Line21 := nil;
  end;
end;

function TSRVMediaPlayer.SetOutputFormatCCText(lpbmih: TBitmapInfoHeader) : Boolean;
var
     Line21 : IAMLine21Decoder;
begin
  Result := False;
  if Succeeded(FilterGraph.QueryInterface(IID_IAMLine21Decoder, Line21)) then begin
    Result := Succeeded(Line21.SetOutputFormat(lpbmih));
    Line21 := nil;
  end;
end;

end.
