unit SRVMemo;

interface
{$I RV_Defs.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, SRVControl, SRVEdit, RVUni, RVScroll, RVItem, RVTypes, RVGrIn;

type
  TSRVMemo = class(TSRVEdit)
  private
    { Private declarations }
    TextLines: TStringList;
    FLines: TStrings;
    FAlignment: TAlignment;
    FScrollBars: TScrollStyle;
    FWordWrap: Boolean;
    FWantReturns: Boolean;
    FWantTabs: Boolean;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMNCDestroy(var Message: TWMNCDestroy); message WM_NCDESTROY;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    { Protected declarations }
    indexRow1 : Integer;
    indexRow2 : Integer;
    iRowSel1 : Integer;
    iRowSel2 : Integer;
    lenFirstRow : Integer;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWindowHandle(const Params: TCreateParams); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure GetBoundPos; override;
    procedure SetAlignment(Value: TAlignment);
    procedure SetLines(Value: TStrings);
    procedure SetScrollBars(Value: TScrollStyle);
    procedure SetWordWrap(Value: Boolean);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function  GetCaretPos: TPoint; override;
    property CaretPos: TPoint read GetCaretPos;
    function GetControlsAlignment: TAlignment; override;
    property Lines: TStrings read FLines write SetLines;
    procedure PaintTo(DC : THandle); override;
  published
    { Published declarations }
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property ScrollBars: TScrollStyle read FScrollBars write SetScrollBars default ssNone;
    property WantReturns: Boolean read FWantReturns write FWantReturns default True;
    property WantTabs: Boolean read FWantTabs write FWantTabs default False;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default True;
  end;

procedure Register;

implementation

uses Consts, Math;

procedure Register;
begin
  RegisterComponents('SRichView', [TSRVMemo]);
end;

type
  TSelection = record
    StartPos, EndPos: Integer;
  end;

  TMemoStrings = class(TStrings)
  private
    Memo: TSRVMemo;
  protected
    function Get(Index: Integer): string; override;
    function GetCount: Integer; override;
    function GetTextStr: string; override;
    procedure Put(Index: Integer; const S: string); override;
    procedure SetTextStr(const Value: string); override;
    procedure SetUpdateState(Updating: Boolean); override;
  public
    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    procedure Insert(Index: Integer; const S: string); override;
  end;

{ TMemoStrings }

function TMemoStrings.GetCount: Integer;
begin
  Result := 0;
  if Memo.HandleAllocated then
  begin
    Result := SendMessage(Memo.Handle, EM_GETLINECOUNT, 0, 0);
    if SendMessage(Memo.Handle, EM_LINELENGTH, SendMessage(Memo.Handle,
      EM_LINEINDEX, Result - 1, 0), 0) = 0 then Dec(Result);
  end;
end;

function TMemoStrings.Get(Index: Integer): string;
var
  Text: array[0..4095] of Char;
begin
  Word((@Text)^) := SizeOf(Text);
  SetString(Result, Text, SendMessage(Memo.Handle, EM_GETLINE, Index,
    LParam(@Text)));
end;

procedure TMemoStrings.Put(Index: Integer; const S: string);
var
  SelStart: Integer;
begin
  SelStart := SendMessage(Memo.Handle, EM_LINEINDEX, Index, 0);
  if SelStart >= 0 then
  begin
    SendMessage(Memo.Handle, EM_SETSEL, SelStart, SelStart +
      SendMessage(Memo.Handle, EM_LINELENGTH, SelStart, 0));
    SendMessage(Memo.Handle, EM_REPLACESEL, 0, LParam(PChar(S)));
    Memo.TextLines.Assign(Self);
  end;
end;

procedure TMemoStrings.Insert(Index: Integer; const S: string);
var
  SelStart, LineLen: Integer;
  Line: string;
begin
  if Index >= 0 then
  begin
    SelStart := SendMessage(Memo.Handle, EM_LINEINDEX, Index, 0);
    if SelStart >= 0 then Line := S + #13#10 else
    begin
      SelStart := SendMessage(Memo.Handle, EM_LINEINDEX, Index - 1, 0);
      if SelStart < 0 then Exit;
      LineLen := SendMessage(Memo.Handle, EM_LINELENGTH, SelStart, 0);
      if LineLen = 0 then Exit;
      Inc(SelStart, LineLen);
      Line := #13#10 + s;
    end;
    SendMessage(Memo.Handle, EM_SETSEL, SelStart, SelStart);
    SendMessage(Memo.Handle, EM_REPLACESEL, 0, LParam(PChar(Line)));
    Memo.TextLines.Assign(Self);
  end;
end;

procedure TMemoStrings.Delete(Index: Integer);
const
  Empty: PChar = '';
var
  SelStart, SelEnd: Integer;
begin
  SelStart := SendMessage(Memo.Handle, EM_LINEINDEX, Index, 0);
  if SelStart >= 0 then
  begin
    SelEnd := SendMessage(Memo.Handle, EM_LINEINDEX, Index + 1, 0);
    if SelEnd < 0 then SelEnd := SelStart +
      SendMessage(Memo.Handle, EM_LINELENGTH, SelStart, 0);
    SendMessage(Memo.Handle, EM_SETSEL, SelStart, SelEnd);
    SendMessage(Memo.Handle, EM_REPLACESEL, 0, LParam(Empty));
    Memo.TextLines.Assign(Self);
  end;
end;

procedure TMemoStrings.Clear;
begin
  Memo.Clear;
  Memo.TextLines.Clear;
end;

procedure TMemoStrings.SetUpdateState(Updating: Boolean);
begin
  if Memo.HandleAllocated then
  begin
    SendMessage(Memo.Handle, WM_SETREDRAW, Ord(not Updating), 0);
    if not Updating then
    begin   // WM_SETREDRAW causes visibility side effects in memo controls
      Memo.Perform(CM_SHOWINGCHANGED,0,0); // This reasserts the visibility we want
      Memo.Refresh;
    end;
    Memo.TextLines.Assign(Self);
  end;
end;

function TMemoStrings.GetTextStr: string;
begin
  Result := Memo.Text;
end;

procedure TMemoStrings.SetTextStr(const Value: string);
var
  NewText: string;
begin
  NewText := AdjustLineBreaks(Value);
  if (Length(NewText) <> Memo.GetTextLen) or (NewText <> Memo.Text) then
  begin
    if SendMessage(Memo.Handle, WM_SETTEXT, 0, LPARAM(NewText)) = 0 then
      raise EInvalidOperation.Create(SInvalidMemoSize);
    Memo.Perform(CM_TEXTCHANGED, 0, 0);
  end;
  Memo.TextLines.Assign(Self);
end;


{ TSRVMemo }

constructor TSRVMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 185;
  Height := 89;
  AutoSize := False;
  FWordWrap := True;
  FWantReturns := True;
  FLines := TMemoStrings.Create;
  TMemoStrings(FLines).Memo := Self;
  TextLines := TStringList.Create;
end;

destructor TSRVMemo.Destroy;
begin
  FLines.Free;
  TextLines.Free;
  inherited Destroy;
end;

procedure TSRVMemo.CreateParams(var Params: TCreateParams);
const
  Alignments: array[Boolean, TAlignment] of DWORD =
    ((ES_LEFT, ES_RIGHT, ES_CENTER),(ES_RIGHT, ES_LEFT, ES_CENTER));
  ScrollBar: array[TScrollStyle] of DWORD = (0, WS_HSCROLL, WS_VSCROLL,
    WS_HSCROLL or WS_VSCROLL);
  WordWraps: array[Boolean] of DWORD = (0, ES_AUTOHSCROLL);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style and not WordWraps[FWordWrap] or ES_MULTILINE or
      Alignments[UseRightToLeftAlignment, FAlignment] or ScrollBar[FScrollBars];
  end;
end;

procedure TSRVMemo.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited;
  exit;
  with Params do
  begin
    if SysLocale.FarEast and (Win32Platform <> VER_PLATFORM_WIN32_NT) and
      ((Style and ES_READONLY) <> 0) then
    begin
      // Work around Far East Win95 API/IME bug.
      WindowHandle := CreateWindowEx(ExStyle, WinClassName, '',
        Style and (not ES_READONLY),
        X, Y, Width, Height, WndParent, 0, HInstance, Param);
      if WindowHandle <> 0 then
        SendMessage(WindowHandle, EM_SETREADONLY, Ord(True), 0);
    end
    else
      WindowHandle := CreateWindowEx(ExStyle, WinClassName, '', Style,
        X, Y, Width, Height, WndParent, 0, HInstance, Param);
    SendMessage(WindowHandle, WM_SETTEXT, 0, LParam(Caption));
  end;
end;

function TSRVMemo.GetCaretPos: TPoint;
begin
  Result := Point(-1, -1);
  if Focused then
    begin
      Windows.GetCaretPos(Result);
      Result.x := Result.x + BorderWidth;
      Result.y := Result.y + BorderWidth;
      if (Result.x > Width-BorderWidth) or (Result.y > Height-BorderWidth) then
        Result := Point(-1, -1);
      TextLines.Assign(FLines);
      GetBoundPos;
    end;
end;

procedure TSRVMemo.GetBoundPos;
var
     tmp, i, len : Integer;
     //f: Boolean;
begin
  // indexPos1 := SendMessage(Handle, EM_CHARFROMPOS, 0, MAKELPARAM(BorderWidth, 1));
  // indexPos2 := SendMessage(Handle, EM_CHARFROMPOS, 0, MAKELPARAM(ClientWidth - 1, 1));

{  indexPos1 := 0;
  indexPos2 := Length(Text)-1;

  f := False;
  for i := 0 to Length(Text)-1 do begin
    tmp := SendMessage(Handle, EM_POSFROMCHAR, i, 0);
    if (tmp and $FF00)<>$FF00 then begin
      if not f then begin
        indexPos1 := i;
        FStartX := tmp and $FF;
        f := True;
        end
      else if f then begin
        if tmp and $FF>ClientWidth then begin
          indexPos2 := i-1;
          break;
        end;
      end;
      end
    else if f then begin
      indexPos2 := i-1;
      break;
    end;
  end;


  iSel1:= 0;
  iSel2:= 0;
  if (SelLength > 0) then begin
    iSel1:= Max(indexPos1, SelStart);
    iSel2:= Min(indexPos2+1, SelStart + SelLength);
    if iSel1 > iSel2 then begin
      tmp := iSel1;
      iSel1 := iSel2;
      iSel2 := tmp;
    end;
  end;}

  indexPos1 := 0;
  indexPos2 := Length(Text)-1;
//  indexRow1 := 0;
//  indexRow2 := TextLines.Count-1;
  indexRow1 := SendMessage(Handle, EM_GETFIRSTVISIBLELINE, 0, 0);
  indexRow2 := LongRec(Integer(SendMessage(Handle, EM_CHARFROMPOS, 0, MAKELPARAM(5, ClientHeight-5)))).Hi;
  indexRow2 := Min(TextLines.Count-1, indexRow2);

  iSel1:= SelStart;
  iSel2:= SelStart + SelLength;
  iRowSel1:= 0;
  iRowSel2:= 0;
  if SelLength > 0 then begin
    iSel1:= SelStart;
    iSel2:= SelStart + SelLength;
    if iSel1 > iSel2 then begin
      tmp := iSel1;
      iSel1 := iSel2;
      iSel2 := tmp;
    end;

    len := 0;
    lenFirstRow := 0;
    for i:=0 to TextLines.Count-1 do begin
      if i = indexRow1 then
        lenFirstRow := len;
      if (iSel1 > len) and
         (iSel1 <= len + Length(TextLines.Strings[i])) then
        iRowSel1 := i;
      if (iSel2 > len) and
         (iSel2 <= len + Length(TextLines.Strings[i])) then begin
        iRowSel2 := i;
        break;
      end;
      len := len + Length(TextLines.Strings[i]);
    end;

    iRowSel1:= Max(indexRow1, iRowSel1);
    iRowSel2:= Min(indexRow2, iRowSel2);
  end;
end;

function TSRVMemo.GetControlsAlignment: TAlignment;
begin
  Result := FAlignment;
end;

procedure TSRVMemo.Loaded;
begin
  inherited Loaded;
  Modified := False;
  TextLines.Assign(FLines);
end;

procedure TSRVMemo.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TSRVMemo.SetLines(Value: TStrings);
begin
  FLines.Assign(Value);
  TextLines.Assign(FLines);
end;

procedure TSRVMemo.SetScrollBars(Value: TScrollStyle);
begin
  if FScrollBars <> Value then
  begin
    FScrollBars := Value;
    RecreateWnd;
  end;
end;

procedure TSRVMemo.SetWordWrap(Value: Boolean);
begin
  if Value <> FWordWrap then
  begin
    FWordWrap := Value;
    RecreateWnd;
  end;
end;

procedure TSRVMemo.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;
  if FWantTabs then Message.Result := Message.Result or DLGC_WANTTAB
  else Message.Result := Message.Result and not DLGC_WANTTAB;
  if not FWantReturns then
    Message.Result := Message.Result and not DLGC_WANTALLKEYS;
end;

procedure TSRVMemo.WMNCDestroy(var Message: TWMNCDestroy);
begin
  inherited;
end;

procedure TSRVMemo.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key = Char(VK_RETURN)) and not FWantReturns then Key := #0;
end;

procedure TSRVMemo.PaintTo(DC : THandle);
var
     editDC, DCScreen: HDC;
     R, rSel: TRect;
     SZ: TSize;
     w, row, offsY, tH, i1, i2, len
     //,FontSize
     : Integer;
     Canvas, CanvasScreen: TCanvas;
     txt, s: String;
     sraw: TRVRawByteString;
     ItemOptions: TRVItemOptions;
     DoSkin : Boolean;
begin
  if (Parent = nil) then
    Exit;

  If ParentSRV and (DC = 0) then begin
    editDC := GetWindowDC(Handle);
    ReleaseDC(Handle, editDC);
    if Parent <> nil then
      PostMessage(Parent.Handle, WM_REPAINT_CONTROL, 0, 0);
    exit;
  end;

  if DC = 0 then
    editDC := GetWindowDC(Handle)
  else
    editDC := DC;

  Canvas := TCanvas.Create;
  CanvasScreen := TCanvas.Create;
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  If RequiredCanvasOff then
    OffsetRect(R, CanvasOffX, CanvasOffY)
  else begin
    CanvasOffX := 0;
    CanvasOffY := 0;
  end;
  DCScreen := CreateCompatibleDC(0);

  try
    Canvas.Handle := editDC;

{    Canvas.Brush.Style := bsSolid;
    Canvas.Brush.Color := Color;
    Canvas.FillRect(R);}

    DrawBoxBG(Canvas, R, SkinManager, EditSkinSchemeIndex, True,
      MouseIn, ClickState <> 0, Enabled, Focused, 1, Color);

    CanvasScreen.Handle := DCScreen;
    CanvasScreen.Font.PixelsPerInch := 96;

    offsY := 0;
    len := lenFirstRow;
    if TextLines.Count > 0 then
    for row:=indexRow1 to indexRow2 do begin

    Canvas.Font.Assign(Font);
    If TextColor <> clNone then
      Canvas.Font.Color := TextColor;
    CanvasScreen.Font.Assign(Canvas.Font);
    Canvas.Brush.Color := Color;

    txt := TextLines.Strings[row];
    s := Copy(txt, indexPos1 + 1, indexPos2 - indexPos1 + 1);

    {$IFDEF RVUNICODESTR}
    sraw := RVU_GetRawUnicode(s);
    ItemOptions := [rvioUnicode];
    {$ELSE}
    sraw := s;
    ItemOptions := [];
    {$ENDIF}
    SZ := CanvasScreen.TextExtent(s);
    if (row = iRowSel1) then
      w := (Width - ClientWidth) div 2 + FStartX
    else
      w := (Width - ClientWidth) div 2;
    If BGTextColor <> clNone then
      Canvas.Brush.Color := BGTextColor;

//    SRVCMultilineTextOut(Canvas, GetFormatCanvas, Round(ClientRect.Left) + w + CanvasOffX, Round(ClientRect.Top) + BorderWidth + CanvasOffY, ClientWidth,
//      Alignment, Text, FGraphicInterface);

    if not RVU_DrawSelectedTextEx(w + CanvasOffX, BorderWidth + CanvasOffY + offsY, SZ.cx, sraw,
      Canvas, CanvasScreen,
      1, Length(s), ItemOptions, rvbdLeftToRight, False, FGraphicInterface) then begin
      //Canvas.Font.Size := FontSize;
      Canvas.Brush.Style := bsClear;
      Canvas.TextRect(R, BorderWidth + CanvasOffX, BorderWidth + CanvasOffY + offsY, s);
      Canvas.Brush.Style := bsSolid;
    end;

    tH :=  Canvas.TextHeight(txt);
    if not Printing and (iSel2 > iSel1) and (row <= iRowSel2) and (row >= iRowSel1) then begin
      if (iRowSel1 = iRowSel2) then begin
        rSel := Rect(w + Canvas.TextWidth(Copy(txt, indexPos1+1, iSel1 - indexPos1)),
                     BorderWidth,
                     w + Canvas.TextWidth(Copy(txt, indexPos1+1, iSel2 - indexPos1)),
                     Min(BorderWidth + tH + offsY, R.Top + tH));
        i1 := -indexPos1+iSel1+1 - len;
        i2 := -indexPos1+iSel2 - len;
      end else if (row = iRowSel1) then begin
        rSel := Rect(w + Canvas.TextWidth(Copy(txt, indexPos1+1, iSel1 - indexPos1)),
                     BorderWidth,
                     w + Canvas.TextWidth(Copy(txt, indexPos1+1, length(txt) - indexPos1)),
                     Min(BorderWidth + tH + offsY, R.Top + tH));
        i1 := -indexPos1+iSel1+1 - len;
        i2 := length(txt);
      end else if (row = iRowSel2) then begin
        rSel := Rect(w,
                     BorderWidth,
                     w + Canvas.TextWidth(Copy(txt, 1, iSel2-indexPos1)),
                     Min(BorderWidth + tH + offsY, R.Top + tH));
        i1 := 1;
        i2 := -len+iSel2;
      end else begin
        rSel := Rect(w,
                     BorderWidth,
                     w + Canvas.TextWidth(txt),
                     Min(BorderWidth + tH + offsY, R.Top + tH));
        i1 := 1;
        i2 := length(txt);
      end;

      Canvas.Brush.Style := bsSolid;
      Canvas.Brush.Color := clHighlight;
      Canvas.Font.Color := clWhite;

      if not RVU_DrawSelectedTextEx(w + CanvasOffX, BorderWidth + CanvasOffY + offsY,
        SZ.cx, sraw, Canvas, CanvasScreen, i1, i2,
        ItemOptions, rvbdLeftToRight, False,
        FGraphicInterface) then begin
        //Canvas.Font.Size := FontSize;
        Canvas.Brush.Style := bsClear;
        Canvas.TextRect(rSel, rSel.Left + CanvasOffX, rSel.Top + CanvasOffY + offsY,
                        Copy(txt, iSel1+1, iSel2 - iSel1 + 1));
        Canvas.Brush.Style := bsSolid;
      end;
    end;
    offsY := offsY + tH;
    len := len + Length(txt);
    end;

    DoSkin := DrawBoxBorder(Canvas, R, FSkinManager, EditSkinSchemeIndex, True,
      MouseIn, ClickState <> 0, Enabled, Focused, 1);

    if not DoSkin then begin
      if Enabled = True then
        if MouseIn then
          Frame3D(Canvas, R, MouseInBorderColor, MouseInBorderColor, MouseInBorderWidth)
        else if Focused then
          Frame3D(Canvas, R, FocusedBorderColor, FocusedBorderColor, FocusedBorderWidth)
        else
          Frame3D(Canvas, R, EnabledBorderColor, EnabledBorderColor, EnabledBorderWidth)
      else
        Frame3D(Canvas, R, DisabledBorderColor, DisabledBorderColor, DisabledBorderWidth);
      If Color <> clNone then
        Frame3D(Canvas, R, Color, Color, 1);
    end;
  finally
    DeleteDC(DCScreen);
    if DC = 0 then
      ReleaseDC(Handle, editDC);
    Canvas.Handle := 0;
    Canvas.Free;
    CanvasScreen.Handle := 0;
    CanvasScreen.Free;
  end;
  CheckListPosition;
end;

procedure TSRVMemo.CMTextChanged(var Message: TMessage);
begin
  inherited;
  TextLines.Assign(FLines);
end;

end.
